import logging
import os

from CIDAN.GUI.Data_Interaction.DataHandler import DataHandler
from CIDAN.GUI.Inputs.FileInput import createFileDialog

logger1 = logging.getLogger("CIDAN.loadDataset")


# def loadImageWrapper(main_widget):
#     def loadImage():
#         path_to_file = createFileDialog("~/Desktop", forOpen=True, fmt='', isFolder=0)
#         path_to_save_dir = createFileDialog("~/Desktop", forOpen=False, fmt='',
#                                             isFolder=1)
#         if hasattr(main_widget, "data_handler"):
#             main_widget.data_handler.__del__()
#         main_widget.data_handler = DataHandler(data_path=path_to_file,
#                                                save_dir_path=path_to_save_dir,
#                                                save_dir_already_created=False)
#         main_widget.image_view.setImage(main_widget.data_handler.calculate_filters())
#
#     return loadImage


def load_new_dataset(main_widget, file_input, save_dir_input, trials=None, single=False):
    print(trials)

    file_path = file_input.current_state()
    save_dir_path = save_dir_input.current_state()
    # if hasattr(main_widget, "data_handler"):
    #     main_widget.data_handler.__del__()
    if single:
        try:
            dir_path = os.path.dirname(file_path)

            main_widget.data_handler = DataHandler(data_path=dir_path,
                                                   trials=[os.path.basename(file_path)],
                                                   save_dir_path=save_dir_path,
                                                   save_dir_already_created=False)
            main_widget.init_w_data()
        except IndentationError as e:
            logger1.error(e)
            print("Loading Failed please make sure it is a valid file")
    elif not trials:
            try:
                dir_path = os.path.dirname(file_path)

                main_widget.data_handler = DataHandler(data_path=dir_path,
                                                       trials=[os.path.basename(file_path)],
                                                       save_dir_path=save_dir_path,
                                                       save_dir_already_created=False)
                main_widget.init_w_data()
            except IndentationError as e:
                logger1.error(e)
                print("Loading Failed please make sure it is a valid file")
    elif trials:
        logger1.debug("Trials:" + str(trials))
        if len(trials) == 0:
            print("Please select at least one trial")
        try:
            main_widget.data_handler = DataHandler(data_path=file_path, trials=trials,
                                                   save_dir_path=save_dir_path,
                                                   save_dir_already_created=False)
            main_widget.init_w_data()
        except IndentationError as e:
            logger1.error(e)
            print("Loading Failed please make sure it is a valid folder and all trials"
                  + " are valid files")


def load_prev_session(main_widget, save_dir_input):
    # TODO add error handling here to ensure valid inputs
    save_dir_path = save_dir_input.current_state()
    try:
        main_widget.data_handler = DataHandler(data_path="",
                                               save_dir_path=save_dir_path,
                                               save_dir_already_created=True)
        main_widget.init_w_data()
    except Exception as e:
        logger1.error(e)
        print("Loading Failed please try again, if problem persists save directory " +
              "is corrupted")


def export_timetraces(main_widget):
    createFileDialog(forOpen=False, isFolder=2)



import os
from typing import Tuple, List

import matplotlib.pyplot as plt
import numpy as np
import tifffile
from PIL import Image
from dask import delayed
from scipy import ndimage


# from IPython.display import display, Image

def load_filter_tif_stack(*, path, filter: bool, median_filter: bool,
                          median_filter_size: Tuple[int, int, int],
                          z_score: bool, slice_stack: bool,
                          slice_every, slice_start: int, crop_stack: bool,
                          crop_x: List[int], crop_y: List[int]):
    """
    This function reads a tiff stack file
    Parameters
    ----------
    path The path to a single tif stack or to a directory of tiff stacks


    Returns
    -------
    a 3D numpy array with the tiff files together

    """
    print(path)

    if os.path.isdir(path):
        volumes = []
        paths = path if type(path) == list else sorted(os.listdir(path))
        print(paths)
        for num, x in enumerate(paths):
            file_path = x if type(path) == list else os.path.join(path, x)
            image = tifffile.imread(file_path)
            if len(image.shape) == 2:
                image = image.reshape((1,image.shape[0],image.shape[1]))
            if slice_stack:
                image = image[slice_start::slice_every]
            if crop_stack:
                image = image[:, crop_x[0]:crop_x[1], crop_y[0]:crop_y[1]]
            if filter:
                image = filter_stack(stack=image, median_filter=median_filter,
                                     median_filter_size=median_filter_size,
                                     z_score=z_score)
            volumes.append(image)
            # print("Loading: " + x)

        image = np.vstack(volumes)
        del volumes
        return image.astype(np.float64)
    if os.path.isfile(path):
        # return ScanImageTiffReader(path).data()
        image = tifffile.imread(path)
        if len(image.shape) == 2:
            image = image.reshape((1, image.shape[0], image.shape[1]))
        if slice_stack:
            image = image[slice_start::slice_every]
        if crop_stack:
            image = image[:, crop_x[0]:crop_x[1], crop_y[0]:crop_y[1]]
        if filter:
            image = filter_stack(stack=image, median_filter=median_filter,
                                 median_filter_size=median_filter_size, z_score=z_score)
        return image.astype(np.float64)
    raise Exception("Invalid Inputs folders not allowed currently ")
    # vol=ScanImageTiffReader(file_path).data()


def reshape_to_2d_over_time(volume):
    """
    Takes a 3d numpy volume with dim 1 and 2 being x, y and dim 0 being time
    and return array compressing dim 1 and 2 into 1 dimension

    Parameters
    ----------
    volume input 3d numpy volume

    Returns
    -------
    a 2d volume with the 0 dim being pixel number and 1 being time value

    """
    return np.transpose(np.reshape(volume, (volume.shape[0], -1), order="C"))


def save_image(volume: np.ndarray, name: str, directory: str, shape: tuple,
               number_save: int):
    """
    Function to save images from a 3d volume
    Parameters
    ----------
    volume
    name
    directory
    shape shape to reshape volume into
    number_save number of slices to take and save

    Returns
    -------
    None
    """
    for x in range(number_save):
        fig, ax = plt.subplots()
        imgplot = ax.imshow(np.reshape(volume,
                                       shape)[shape[0] // number_save * x])
        fig.savefig(os.path.join(directory, name + "_" + str(x)))

        # img = Image.fromarray(
        #     np.reshape(volume,
        #                shape)[shape[0]/number_save*x] * 255).convert('L')
        # img.save(
        #     os.path.join(directory,name+"_"+str(x)))


def filter_stack(*, stack: np.ndarray, median_filter: bool,
                 median_filter_size: Tuple[int, int, int],
                 z_score: bool):
    if median_filter:
        # stack = ndimage.median_filter(stack, median_filter_size)
        stack = ndimage.filters.convolve(stack, np.full((3, 3, 3), 1.0 / 27))
    if z_score:
        stack_t = np.transpose(stack, (1, 2, 0))
        shape = (stack.shape[1], stack.shape[2], 1)
        std = np.std(stack_t, axis=2).reshape(shape)
        mean = np.mean(stack_t, axis=2).reshape(shape)
        stack_t = (stack_t - mean) / std
        stack = np.transpose(stack_t, (2, 0, 1))

    return stack


def pixel_num_to_2d_cord(pixel_list, volume_shape: Tuple[int, int]):
    """
    Takes a list of pixels and converts it to a list of cords for each pixel

    Parameters
    ----------
    pixel_list : np.ndarray
        list of pixel numbers
    volume_shape : tuple
        shape of the original dataset [x,y]

    Returns
    -------
    a list of tuples where each is the cordinates for each pixel

    """

    def convert_one(num):
        return num % volume_shape[1], num // volume_shape[1]

    return np.apply_along_axis(convert_one, axis=0, arr=pixel_list)


@delayed
def join_data_list(data_list):
    """
    Joins a list of trials with the same dimensions
    Parameters
    ----------
    data_list
        Data to stack

    Returns
    -------
    List of data stacked over 1st dimension
    """
    return np.vstack(data_list)


@delayed
def saveTempImage(data, save_dir, spatial_box_num):
    print(save_dir)
    temp_dir = os.path.join(save_dir, "temp_images")
    print(data[60, :, :].shape)
    img = Image.fromarray(data[3, :, :] * 255).convert('L')
    image_path = os.path.join(temp_dir, "embedding_norm_image_box_{}.png".format(
        spatial_box_num))
    img.save(image_path)
    return data



from qtpy.QtWidgets import *

from CIDAN.GUI.Data_Interaction.loadDataset import *
from CIDAN.GUI.Inputs.BoolInput import BoolInput
from CIDAN.GUI.Inputs.FileInput import FileInput
from CIDAN.GUI.ListWidgets.TrialListWidget import TrialListWidget
from CIDAN.GUI.Tabs.Tab import Tab


class FileOpenTab(Tab):
    def __init__(self, main_widget):
        # TODO Make this less ugly can reorganize code
        self.trial_list_widget = TrialListWidget()
        dataset_file_input = FileInput("Dataset File:", "", None, "",
                                       "Select a file to load in", isFolder=2,
                                       forOpen=True)
        dataset_folder_input = FileInput("Dataset Folder:", "",
                                         lambda x: self.add_trial_selection(x), "",
                                         "Select a folder to load in", isFolder=1,
                                         forOpen=True)
        save_dir_new_file = FileInput("Save Directory Location:", "", None, "",
                                      "Select a place to save outputs", isFolder=1,
                                      forOpen=False)
        save_dir_new_folder = FileInput("Save Directory Location:", "", None, "",
                                        "Select a place to save outputs", isFolder=1,
                                        forOpen=False)

        save_dir_load = FileInput("Previous Session Location:", "", None, "",
                                  tool_tip="Select the save directory for a previous session",
                                  isFolder=1, forOpen=True)
        file_open_button = QPushButton()
        file_open_button.setContentsMargins(0, 0, 0, 11)
        file_open_button.setText("Load")
        file_open_button.clicked.connect(
            lambda: load_new_dataset(main_widget, dataset_file_input,
                                     save_dir_new_file))
        folder_open_button = QPushButton()
        folder_open_button.setContentsMargins(0, 0, 0, 11)
        folder_open_button.setText("Load")
        folder_open_button.clicked.connect(
            lambda: load_new_dataset(main_widget, dataset_folder_input,
                                     save_dir_new_folder,
                                     trials=self.trial_list_widget.selectedTrials(), single=self.folder_open_single_trial.current_state()))
        self.folder_open_single_trial = BoolInput("Open folder as single trial","",None,False,"")
        prev_session_open_button = QPushButton()
        prev_session_open_button.setContentsMargins(0, 0, 0, 11)
        prev_session_open_button.setText("Load")
        prev_session_open_button.clicked.connect(
            lambda: load_prev_session(main_widget, save_dir_load))
        file_open = Tab("File Open", column_2=[], column_2_display=False,
                        column_1=[dataset_file_input, save_dir_new_file,
                                  file_open_button])
        folder_open = Tab("Folder Open", column_2=[self.trial_list_widget],
                          column_2_display=True,
                          column_1=[dataset_folder_input, save_dir_new_folder,self.folder_open_single_trial,
                                    folder_open_button]
                          )
        prev_session_open = Tab("Previous Session Open", column_2=[],
                                column_2_display=False,
                                column_1=[save_dir_load, prev_session_open_button])
        self.tab_selector = QTabWidget()

        self.tab_selector.addTab(file_open, file_open.name)
        self.tab_selector.addTab(folder_open, folder_open.name)
        self.tab_selector.addTab(prev_session_open, prev_session_open.name)

        super().__init__("FileOpenTab", column_1=[self.tab_selector], column_2=[],
                         column_2_display=False)

    def add_trial_selection(self, path):
        self.trial_list_widget.set_items_from_path(path)



import logging

from CIDAN.GUI.Data_Interaction.Signals import *
from CIDAN.GUI.Data_Interaction.Thread import Thread

logger = logging.getLogger("CIDAN.GUI.Data_Interaction.ROIExctractionThread")
class ROIExtractionThread(Thread):
    def __init__(self, main_widget, button, roi_list_module, roi_tab):
        super().__init__(main_widget.data_handler)
        self.roi_tab = roi_tab
        self.main_widget = main_widget
        self.roi_list_module = roi_list_module
        self.button = button
        self.signal.sig.connect(lambda x: self.endThread(x))

    def run(self):
        try:
            self.data_handler.calculate_roi_extraction()
            self.signal.sig.emit(True)
        except AssertionError as e:
            if (type(e) == AssertionError):
                print(e.args[0])
            else:
                print("Something weird happened please reload and try again")
            logger.error(e)
            self.signal.sig.emit(False)

    def runThread(self):

        if not any([x.isRunning() for x in self.main_widget.thread_list]):
            print("Starting ROI extraction")
            # self.button.setEnabled(False)
            self.start()
        else:
            print(
                "Previous process in process, please wait to start new one till finished")

    def endThread(self, success):
        self.button.setEnabled(True)
        if success:
            print("Finished ROI extraction")
            self.roi_list_module.set_list_items(self.main_widget.data_handler.clusters)
            shape = self.main_widget.data_handler.edge_roi_image_flat.shape
            self.roi_tab.roi_image_flat = np.hstack(
                [self.main_widget.data_handler.edge_roi_image_flat,
                 np.zeros(shape),
                 np.zeros(shape)])
            self.roi_tab.select_image_flat = np.zeros([shape[0], 3])

            self.roi_tab.updateImageDisplay(new=True)



from CIDAN.GUI.Data_Interaction.Signals import *


class Thread(QThread):
    def __init__(self, data_handler, parent=None):
        QThread.__init__(self, parent)
        self.exiting = False
        self.signal = BoolSignal()
        self.data_handler = data_handler

    def run(self):
        pass

    def endThread(self, success):
        pass



from qtpy import QtCore
from qtpy.QtGui import *
from qtpy.QtWidgets import *


class ROIItemWidget(QWidget):
    def __init__(self, roi_tab, color, roi_list, roi_num, parent=None):
        self.roi_tab = roi_tab
        self.roi_list = roi_list
        self.roi_num = roi_num
        super(ROIItemWidget, self).__init__(parent)
        self.setStyleSheet("""QPushButton {background-color: rgba(0,0,0,0%);
        padding-left:3px;
        padding-right:3px;
        
        color: #CCCCCC;}
        QPushButton:hover {
          border: 1px solid #148CD2;
          background-color: #505F69;
          color: #F0F0F0;
            }
            QPushButton:pressed {
  background-color: #19232D;
  border: 1px solid #19232D;
}

QPushButton:pressed:hover {
  border: 1px solid #148CD2;
}
QPushButton:selected {
  background-color: rgba(0,0,0,0%);
  color: #32414B;
}
            QLabel {
            background-color: rgba(0,0,0,0%)
            }QCheckBox {
            background-color: rgba(0,0,0,0%)
            }""")
        self.zoom_button = QPushButton("Zoom To")
        self.zoom_button.clicked.connect(lambda x: self.roi_tab.zoomRoi(self.roi_num))
        # self.button = QtGui.QPushButton("Selection Add")
        # self.button.clicked.connect(lambda x: self.roi_tab.modify_roi(self.roi_num, "add"))
        # self.button2 = QtGui.QPushButton("Selection Subtract")
        # self.button2.clicked.connect(
        #     lambda x: self.roi_tab.modify_roi(self.roi_num, "subtract"))
        # self.button3 = QtGui.QPushButton("Delete ROI")
        self.check_box = QCheckBox()
        self.check_box.toggled.connect(lambda: self.check_box_toggled())
        self.check_box_time_trace = QCheckBox()
        self.check_box_time_trace.toggled.connect(lambda: self.time_check_box_toggled())
        out_img = QImage(100, 100, QImage.Format_ARGB32)
        out_img.fill(Qt.transparent)
        brush = QBrush(QColor(*color))  # Create texture brush
        painter = QPainter(out_img)  # Paint the output image
        painter.setBrush(brush)  # Use the image texture brush
        painter.setPen(Qt.NoPen)  # Don't draw an outline
        painter.setRenderHint(QPainter.Antialiasing, True)  # Use AA
        painter.drawEllipse(0, 0, 100, 100)  # Actually draw the circle
        painter.end()  # We are done (segfault if you forget this)

        # Convert the image to a pixmap and rescale it.  Take pixel ratio into
        # account to get a sharp image on retina displays:
        pr = QWindow().devicePixelRatio()
        pm = QPixmap.fromImage(out_img)
        label_pix = QLabel()
        label_pix.setPixmap(pm)
        label_pix.setMaximumWidth(10)
        lay = QHBoxLayout(self)
        lay.addWidget(self.check_box, alignment=QtCore.Qt.AlignLeft)
        # lay.addWidget(label_pix, alignment=QtCore.Qt.AlignLeft)
        lay.addWidget(QLabel(text="#" + str(roi_num)), alignment=QtCore.Qt.AlignLeft)
        lay.addWidget(QLabel())
        lay.addWidget(QLabel())
        lay.addWidget(QLabel())
        lay.addWidget(
            QLabel(str(round(self.roi_tab.data_handler.roi_circ_list[roi_num - 1], 3))))
        lay.addWidget(self.zoom_button, alignment=QtCore.Qt.AlignRight)
        lay.addWidget(self.check_box_time_trace, alignment=QtCore.Qt.AlignRight)
        # lay.addWidget(self.button, alignment=QtCore.Qt.AlignRight)
        # lay.addWidget(self.button2, alignment=QtCore.Qt.AlignRight)
        # lay.addWidget(self.button3, alignment=QtCore.Qt.AlignRight)
        lay.setContentsMargins(0, 0, 0, 0)

    def select_check_box(self):
        if not self.check_box.checkState():
            for x in self.roi_list.roi_item_list:
                if x != self:
                    x.check_box.setChecked(False)
            self.check_box.setChecked(True)
            self.roi_list.current_selected_roi = self.roi_num

        else:
            self.check_box.setChecked(False)
            self.roi_list.current_selected_roi = None

    def select_time_check_box(self):
        self.check_box_time_trace.setChecked(not self.check_box_time_trace.checkState())

    def check_box_toggled(self):
        if self.check_box.checkState():

            self.roi_tab.selectRoi(self.roi_num)

            for x in self.roi_list.roi_item_list:
                if x != self:
                    x.check_box.setChecked(False)

            self.roi_list.current_selected_roi = self.roi_num
        else:
            self.roi_list.current_selected_roi = None
            self.roi_tab.deselectRoi(self.roi_num)

    def time_check_box_toggled(self):
        self.roi_list.roi_time_check_list[
            self.roi_num - 1] = self.check_box_time_trace.checkState()
        if self.check_box_time_trace.checkState():
            self.roi_tab.selectRoiTime(self.roi_num)
        else:
            self.roi_tab.deselectRoiTime(self.roi_num)



import os

import matplotlib.pyplot as plt
import numpy as np

from CIDAN.LSSC.functions import data_manipulation


def save_volume_images(volume, output_dir):
    data_manipulation.save_image(volume,
                                 "original_image_filtered",
                                 output_dir,
                                 volume.shape,
                                 number_save=4)
    data_manipulation.save_image(np.max(volume, axis=0),
                                 "max_intensity_filtered",
                                 output_dir, (
                                     1, volume.shape[1],
                                     volume.shape[2]),
                                 number_save=1)
    data_manipulation.save_image(np.mean(volume, axis=0),
                                 "mean_intensity_filtered",
                                 output_dir,
                                 (1, volume.shape[1],
                                  volume.shape[2]),
                                 number_save=1)


def save_eigen_images(eigen_vectors, output_dir, image_shape, box_num=0):
    e_vectors_squared = np.power(eigen_vectors, 2)

    e_vectors_reshape = np.transpose(
        np.reshape(e_vectors_squared, (
            image_shape[1],
            image_shape[2], eigen_vectors.shape[1],),
                   order="C"), (2, 0, 1))
    data_manipulation.save_image(e_vectors_reshape,
                                 "box_{}_eigen_vectors".format(str(box_num).zfill(2)),
                                 output_dir,
                                 e_vectors_reshape.shape,
                                 number_save=eigen_vectors.shape[1])


def save_roi_images(roi_list, image_shape, output_dir, box_num=0):
    pixel_length = image_shape[1] * image_shape[2]
    original_zeros_all = np.zeros((pixel_length))
    for num, x in enumerate(roi_list):
        original_zeros = np.zeros((pixel_length))
        original_zeros_all[x] = 255
        original_zeros[x] = 255
        imgplot = plt.imshow(np.reshape(original_zeros,
                                        (image_shape[1],
                                         image_shape[2])))
        plt.savefig(os.path.join(output_dir, "box_{}_roi_{}".format(
            str(box_num).zfill(2), str(num).zfill(3)) + "_" + str(0)))

    data_manipulation.save_image(original_zeros_all, "box_{}_roi_all".format(
        str(box_num).zfill(2)),
                                 output_dir, (
                                     1, image_shape[1],
                                     image_shape[2]), 1)



import argparse
import logging
import sys

from qtpy.QtWidgets import QApplication

from CIDAN.GUI import MainWindow


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-d", "--dev", type=bool, default=False, required=False,
                        help="Enable Developement mode")
    LOG_FILENAME = 'log.out'
    parser.add_argument("-lp", "--logpath", type=str, default=LOG_FILENAME,
                        required=False, help="Path to save log file")
    parser.add_argument("-ll", "--loglevel", type=str, default="error",
                        required=False,
                        choices=['debug', 'info', 'warning', 'error', 'critical'])
    args = parser.parse_args()
    LEVELS = {'debug': logging.DEBUG,
              'info': logging.INFO,
              'warning': logging.WARNING,
              'error': logging.ERROR,
              'critical': logging.CRITICAL}

    level = LEVELS.get(args.loglevel, logging.NOTSET)
    logging.basicConfig(filename=args.logpath, level=level)
    logger = logging.getLogger("CIDAN")
    logger.debug("Program started")
    dev = False
    if len(sys.argv) > 1:
        dev = True if sys.argv[1] == "True" else False
    print(dev)
    app = QApplication([])
    app.setApplicationName("CIDAN")
    widget = MainWindow.MainWindow(dev=dev)

    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



import numpy as np
from qtpy.QtCore import *


class MatrixSignal(QObject):
    def __init__(self):
        super().__init__()

    sig = Signal(np.ndarray)


class StrSignal(QObject):
    def __init__(self):
        super().__init__()

    sig = Signal(str)


class BoolSignal(QObject):
    def __init__(self):
        super().__init__()

    sig = Signal(bool)



from qtpy.QtWidgets import *

from CIDAN.GUI.Inputs.BoolInput import BoolInput
from CIDAN.GUI.Inputs.FloatInput import FloatInput
from CIDAN.GUI.Inputs.IntInput import IntInput
from CIDAN.GUI.Inputs.IntRangeInput import IntRangeInput


class SettingBlockModule(QFrame):
    def __init__(self, name, input_list):
        super().__init__()
        self.name = name
        self.input_list = input_list
        self.layout = QVBoxLayout()
        # Type options are int, float, bool, and str
        for input in input_list:
            self.layout.addWidget(input)

        self.setLayout(self.layout)


def filter_setting_block(main_widget):
    data_handler = main_widget.data_handler
    return SettingBlockModule("Filter Settings",
                              [BoolInput(display_name="Median filter:",
                                         program_name="median_filter",
                                         on_change_function=lambda x,
                                                                   y: data_handler.change_filter_param(
                                             x, y),
                                         default_val=data_handler.filter_params[
                                             "median_filter"],
                                         tool_tip="Whether to apply a median filter to each timestep",
                                         ),
                               IntInput(
                                   display_name="Median filter size:",
                                   program_name="median_filter_size",
                                   on_change_function=lambda x,
                                                             y: data_handler.change_filter_param(
                                       x, y),
                                   default_val=
                                   data_handler.filter_params[
                                       "median_filter_size"],
                                   tool_tip="The size of the median filter for each timestep",
                                   min=1, max=50, step=1),
                               BoolInput(
                                   display_name="Z-score:",
                                   program_name="z_score",
                                   on_change_function=lambda x,
                                                             y: data_handler.change_filter_param(
                                       x, y),
                                   default_val=
                                   data_handler.filter_params["z_score"],
                                   tool_tip="Whether to apply a z-score for each pixel across all the timesteps",
                               )
                               ])


def dataset_setting_block(main_widget):
    data_handler = main_widget.data_handler
    return SettingBlockModule("Advanced Settings",
                              [BoolInput(display_name="Slice stack:",
                                         program_name="slice_stack",
                                         on_change_function=lambda x,
                                                                   y: data_handler.change_dataset_param(
                                             x, y),
                                         default_val=data_handler.dataset_params[
                                             "slice_stack"],
                                         tool_tip="Used to Select only "
                                                  "every x timestep",
                                         display_tool_tip=False),
                               IntInput(display_name="Slice every:",
                                        program_name="slice_every",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_dataset_param(
                                            x, y),
                                        default_val=data_handler.dataset_params[
                                            "slice_every"],
                                        tool_tip="Select every x timesteps",
                                        min=1, max=100, step=1),
                               IntInput(display_name="Slice start:",
                                        program_name="slice_start",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_dataset_param(
                                            x, y),
                                        tool_tip="Start selecting on x timestep",
                                        default_val=data_handler.dataset_params[
                                            "slice_start"],
                                        min=0, max=10000, step=1)
                               ])


def dataset_setting_block_crop(main_widget):
    data_handler = main_widget.data_handler
    return SettingBlockModule("Crop Settings",
                              [BoolInput(display_name="Crop stack:",
                                         program_name="crop_stack",
                                         on_change_function=lambda x,
                                                                   y: data_handler.change_dataset_param(
                                             x, y),
                                         default_val=data_handler.dataset_params[
                                             "crop_stack"],
                                         tool_tip="Used to crop image stack",
                                         display_tool_tip=False),
                               IntRangeInput(display_name="Crop X:",
                                             program_name="crop_x",
                                             on_change_function=lambda x,
                                                                       y: data_handler.change_dataset_param(
                                                 x, y),
                                             default_val=data_handler.dataset_params[
                                                 "crop_x"],
                                             tool_tip="Crop in x direction",
                                             min=0, max=100000, step=1),
                               IntRangeInput(display_name="Crop Y:",
                                             program_name="crop_y",
                                             on_change_function=lambda x,
                                                                       y: data_handler.change_dataset_param(
                                                 x, y),
                                             default_val=data_handler.dataset_params[
                                                 "crop_y"],
                                             tool_tip="Crop in y direction",
                                             min=0, max=10000, step=1),
                               # IntRangeInput(display_name="Crop Z:",
                               #               program_name="crop_z",
                               #               on_change_function=lambda x,
                               #                                         y: data_handler.change_dataset_param(
                               #                   x, y),
                               #               default_val=data_handler.dataset_params[
                               #                   "crop_z"],
                               #               tool_tip="Note this crops it for each trial",
                               #               display_tool_tip=True,
                               #               min=data_handler.dataset_params[
                               #                   "crop_z"][0], max=data_handler.dataset_params[
                               #                   "crop_z"][1], step=1)
                               ])

def multiprocessing_settings_block(main_widget):
    data_handler = main_widget.data_handler
    return SettingBlockModule("Multiprocessing Settings", [

        IntInput(
            display_name="Number of spatial boxes:",
            program_name="total_num_spatial_boxes",
            on_change_function=lambda x, y: data_handler.change_box_param(
                x, y),
            default_val=
            data_handler.box_params[
                "total_num_spatial_boxes"],
            tool_tip="Number of boxes to break the calculation into, please make sure it is a sqaure",
            min=1, max=10000, step=1)
        ,
        IntInput(
            display_name="Spatial overlap:",
            program_name="spatial_overlap",
            on_change_function=lambda x, y: data_handler.change_box_param(x, y),
            default_val=
            data_handler.box_params[
                "spatial_overlap"],
            tool_tip="Number of pixels to overlap each box",
            min=1, max=10000, step=1)])


def roi_extraction_settings_block(main_widget):
    # TODO fix input into spatial box number
    data_handler = main_widget.data_handler
    return SettingBlockModule("ROI Extraction Settings",
                              [
                                  IntInput(
                                      display_name="Max Number of ROIs per spatial box:",
                                      program_name="num_rois",
                                      on_change_function=lambda x,
                                                                y: data_handler.change_roi_extraction_param(
                                          x, y),
                                      default_val=data_handler.roi_extraction_params[
                                          "num_rois"],
                                      tool_tip="Max number of ROIs to select for each spatial box",
                                      min=0, max=10000, step=1),
                                  IntInput(
                                      display_name="ROI size minimum:",
                                      program_name="roi_size_min",
                                      on_change_function=lambda x,
                                                                y: data_handler.change_roi_extraction_param(
                                          x, y),
                                      default_val=
                                      data_handler.roi_extraction_params[
                                          "roi_size_min"],
                                      tool_tip="Minimum size in pixels for a region of interest",
                                      min=1, max=10000, step=1)
                                  ,
                                  IntInput(
                                      display_name="ROI size maximum:",
                                      program_name="roi_size_max",
                                      on_change_function=lambda x,
                                                                y: data_handler.change_roi_extraction_param(
                                          x, y),
                                      default_val=
                                      data_handler.roi_extraction_params[
                                          "roi_size_max"],
                                      tool_tip="Maximum size in pixels for a region of interest",
                                      min=1, max=1000000, step=1),
                                  # BoolInput(
                                  #     display_name="Run refinement step:",
                                  #     program_name="refinement",
                                  #     on_change_function=lambda x,
                                  #                               y: data_handler.change_roi_extraction_param(
                                  #         x, y),
                                  #     default_val=
                                  #     data_handler.roi_extraction_params[
                                  #         "refinement"],
                                  #     tool_tip="Whether to run the refinement step, greatly improves results"),
                                  FloatInput(
                                      display_name="Merge temporal coefficient:",
                                      program_name="merge_temporal_coef",
                                      on_change_function=lambda x,
                                                                y: data_handler.change_roi_extraction_param(
                                          x, y),
                                      default_val=
                                      data_handler.roi_extraction_params[
                                          "merge_temporal_coef"],
                                      tool_tip="The coefficient that determines if two overlapping regions are merged based on their temporal correlation",
                                      min=0, max=1, step=.01)
                                  #     BoolInput(
                                  #         display_name="Fill holes:",
                                  #         program_name="fill_holes",
                                  #         on_change_function=lambda x,
                                  #                                   y: data_handler.change_roi_extraction_param(
                                  #             x, y),
                                  #         default_val=
                                  #         data_handler.roi_extraction_params[
                                  #             "fill_holes"],
                                  #         tool_tip="Whether to fill holes in each cluster"),
                                  #
                              ]
                              )


def roi_advanced_settings_block(main_widget):
    # TODO fix input into spatial box number
    data_handler = main_widget.data_handler
    return SettingBlockModule("Advanced Settings",
                              [IntInput(display_name="Number of eigen vectors:",
                                        program_name="num_eig",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_eigen_param(
                                            x, y),
                                        default_val=data_handler.eigen_params[
                                            "num_eig"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),
                               FloatInput(display_name="Eigen threshold value:",
                                          program_name="eigen_threshold_value",
                                          on_change_function=lambda x,
                                                                    y: data_handler.change_roi_extraction_param(
                                              x, y),
                                          default_val=
                                          data_handler.roi_extraction_params[
                                              "eigen_threshold_value"],
                                          tool_tip="Number of eigen vectors to select at each point",
                                          min=0, max=1, step=.01),
                               FloatInput(display_name="Elbow threshold value:",
                                          program_name="elbow_threshold_value",
                                          on_change_function=lambda x,
                                                                    y: data_handler.change_roi_extraction_param(
                                              x, y),
                                          default_val=
                                          data_handler.roi_extraction_params[
                                              "elbow_threshold_value"],
                                          tool_tip="Number of eigen vectors to select at each point",
                                          min=0, max=1.5, step=.01),
                               BoolInput(
                                   display_name="Merge ROIs:",
                                   program_name="merge",
                                   on_change_function=lambda x,
                                                             y: data_handler.change_roi_extraction_param(
                                       x, y),
                                   default_val=
                                   data_handler.roi_extraction_params[
                                       "merge"],
                                   tool_tip="Whether to merge rois with similar time traces"),
                               FloatInput(display_name="ROI Circuity Threshold:",
                                          program_name="roi_circ_threshold",
                                          on_change_function=lambda x,
                                                                    y: data_handler.change_roi_extraction_param(
                                              x, y),
                                          default_val=
                                          data_handler.roi_extraction_params[
                                              "roi_circ_threshold"],
                                          tool_tip="Thresholds the rois based on how circular they are",
                                          min=0, max=1.0, step=.01),
                               IntInput(display_name="Number of connections:",
                                        program_name="connections",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_eigen_param(
                                            x, y),
                                        default_val=data_handler.eigen_params[
                                            "connections"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),
                               IntInput(display_name="accuracy:",
                                        program_name="accuracy",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_eigen_param(
                                            x, y),
                                        default_val=data_handler.eigen_params[
                                            "accuracy"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),
                               IntInput(display_name="Number of knn:",
                                        program_name="knn",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_eigen_param(
                                            x, y),
                                        default_val=data_handler.eigen_params[
                                            "knn"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),
                               IntInput(display_name="Normalize w k :",
                                        program_name="normalize_w_k",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_eigen_param(
                                            x, y),
                                        default_val=data_handler.eigen_params[
                                            "normalize_w_k"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),
                               IntInput(display_name="max iter:",
                                        program_name="max_iter",
                                        on_change_function=lambda x,
                                                                  y: data_handler.change_roi_extraction_param(
                                            x, y),
                                        default_val=data_handler.roi_extraction_params[
                                            "max_iter"],
                                        tool_tip="Number of eigen vectors to generate",
                                        min=1, max=10000, step=1),

                               IntInput(display_name="Number of time steps:",
                                        program_name="total_num_time_steps",
                                        on_change_function=lambda x,
                                        y: data_handler.change_box_param(
                                            x, y),
                                        default_val=data_handler.box_params[
                                            "total_num_time_steps"],
                                        tool_tip=
                                        "Number of time steps to break"+
                                        "the processing into",
                                        min=1, max=10000, step=1),
                               #
                               ]
                              )
# TODO add fill holes/number rois per spatial box to inputs



from pyqtgraph import ImageView
from qtpy.QtWidgets import *


class ImageViewModule(QFrame):
    def __init__(self, main_widget, histogram=True, roi=True):
        super().__init__()

        self.main_window = main_widget
        # self.setMinimumWidth(600)
        # self.setMinimumHeight(300)
        # self.setStyleSheet("ImageViewModule {margin:5px; border:1px solid rgb(50, 65, "
        #                    "75);} ")
        self.setStyleSheet("ImageViewModule {margin:0px; border:0px  solid rgb(50, 65, "
                           "75); padding: 0px;} ")
        self.layout = QHBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)

        # self.layout.setAlignment(Qt.AlignHCenter)

        self.setLayout(self.layout)
        # self.already_loaded = True
        # self.no_image_message = QPushButton("Please open a dataset first")
        # self.no_image_message.clicked.connect(main_widget.open_file_dialog)
        # self.no_image_message.setStyleSheet("QPushButton {font-size:80;}")
        self.image_view = ImageView()
        self.image_view.ui.layoutWidget.setContentsMargins(0, 0, 0, 0)
        # self.image_view.ui.roiBtn.hide()
        self.image_view.ui.menuBtn.hide()
        if not histogram:
            self.image_view.ui.histogram.hide()
        if not roi:
            self.image_view.ui.roiBtn.hide()
        # self.image_view.getRoiPlot().hide()

        self.layout.addWidget(self.image_view)

    def setImage(self, data):

        # if self.already_loaded == False:
        #     print("changed image")
        #     self.already_loaded = True
        #     self.layout.removeWidget(self.no_image_message)
        #     self.no_image_message.deleteLater()
        #     # self.layout.setAlignment(Qt.AlignLeft)
        #     self.image_view = ImageView()
        #
        #     self.layout.addWidget(self.image_view)
        self.image_view.setImage(data, levelMode='mono', autoRange=True,
                                 autoLevels=True, autoHistogramRange=True)



from qtpy import QtCore
from qtpy.QtGui import *


class ROIItemModule(QStandardItem):
    def __init__(self, color, num, roi_tab):
        self.roi_tab = roi_tab
        out_img = QImage(100, 100, QImage.Format_ARGB32)
        out_img.fill(Qt.transparent)
        brush = QBrush(QColor(*color))  # Create texture brush
        painter = QPainter(out_img)  # Paint the output image
        painter.setBrush(brush)  # Use the image texture brush
        painter.setPen(Qt.NoPen)  # Don't draw an outline
        painter.setRenderHint(QPainter.Antialiasing, True)  # Use AA
        painter.drawEllipse(0, 0, 100, 100)  # Actually draw the circle
        painter.end()  # We are done (segfault if you forget this)

        # Convert the image to a pixmap and rescale it.  Take pixel ratio into
        # account to get a sharp image on retina displays:
        pr = QWindow().devicePixelRatio()
        pm = QPixmap.fromImage(out_img)
        self.num = num
        super().__init__(pm, "")
        self.setEditable(False)
        # self.setCheckable(True)

    def toggle_check_state(self):
        if not self.checkState():
            self.roi_tab.selectRoi(self.num)
            self.setCheckState(QtCore.Qt.CheckState.Checked)
        else:
            self.roi_tab.deselectRoi(self.num)
            self.setCheckState(QtCore.Qt.CheckState.Unchecked)

    def checkState(self):
        state = super().checkState()
        if state == QtCore.Qt.CheckState.Unchecked:
            return False
        else:
            return True



import logging
from functools import reduce
from typing import List

import dask
import numpy as np
from dask import delayed

from CIDAN.LSSC.SpatialBox import SpatialBox
from CIDAN.LSSC.functions.data_manipulation import reshape_to_2d_over_time, \
    join_data_list
from CIDAN.LSSC.functions.eigen import generateEigenVectors, saveEigenVectors, \
    loadEigenVectors, saveEmbedingNormImage, createEmbedingNormImageFromMultiple
from CIDAN.LSSC.functions.embeddings import calcAffinityMatrix
from CIDAN.LSSC.functions.roi_extraction import roi_extract_image, merge_rois
from CIDAN.LSSC.functions.save_test_images import save_roi_images

logger1 = logging.getLogger("CIDAN.LSSC.process_data")


def process_data(*, num_threads: int, test_images: bool, test_output_dir: str,
                 save_dir: str, save_intermediate_steps: bool,

                 image_data: List[np.ndarray],
                 eigen_vectors_already_generated: bool,
                 save_embedding_images: bool,
                 total_num_time_steps: int, total_num_spatial_boxes: int,
                 spatial_overlap: int, metric: str, knn: int,
                 accuracy: int, connections: int, normalize_w_k: int, num_eig=25,
                 merge: bool,
                 num_rois: int, refinement: bool, num_eigen_vector_select: int,
                 max_iter: int, roi_size_min: int, fill_holes: bool,
                 elbow_threshold_method: bool, elbow_threshold_value: float,
                 eigen_threshold_method: bool,
                 eigen_threshold_value: float, merge_temporal_coef: float,
                 roi_size_max: int):
    logger1.debug("""Inputs: num_threads {0},test_images {1}, test_output_dir {2},
                 save_dir {3}, save_intermediate_steps {4},
                 load_data {5}, data_path {6},
                 image_data {7},
                 eigen_vectors_already_generated {8},
                 save_embedding_images {9},
                 total_num_time_steps {10}, total_num_spatial_boxes {11},
                 spatial_overlap {12}, filter {13}, median_filter {14},
                 median_filter_size {15},
                 z_score {16}, slice_stack {17},
                 slice_every {18}, slice_start {19}, metric {20}, knn {21},
                 accuracy {22}, connections {23}, normalize_w_k {24}, num_eig {25},
                 merge {26},
                 num_rois {27}, refinement {28}, num_eigen_vector_select {29},
                 max_iter {30}, roi_size_min {31}, fill_holes {32},
                 elbow_threshold_method {33}, elbow_threshold_value {34},
                 eigen_threshold_method {35},
                 eigen_threshold_value {36}, merge_temporal_coef {37},
                 roi_size_max {38}""".format(num_threads, test_images, test_output_dir,
                                             save_dir, save_intermediate_steps,
                                             "", "",
                                             image_data,
                                             eigen_vectors_already_generated,
                                             save_embedding_images,
                                             total_num_time_steps,
                                             total_num_spatial_boxes,
                                             spatial_overlap, "", "", "", "", "", "",
                                             "",
                                             "", metric, knn,
                                             accuracy, connections, normalize_w_k,
                                             num_eig,
                                             merge,
                                             num_rois, refinement,
                                             num_eigen_vector_select,
                                             max_iter, roi_size_min, fill_holes,
                                             elbow_threshold_method,
                                             elbow_threshold_value,
                                             eigen_threshold_method,
                                             eigen_threshold_value, merge_temporal_coef,
                                             roi_size_max))
    # TODO Make after eigen vector make function to save intermediate embeding norm
    #  for each spatial box
    # TODO Rewrite to take in a list of loaded datasets

    # TODO add assertions to make sure input splits work for dataset

    shape = [image_data[0].shape[1], image_data[0].shape[2]]
    logger1.debug("image shape {0}".format(shape))
    print("Creating {} spatial boxes".format(total_num_spatial_boxes))
    spatial_boxes = [SpatialBox(box_num=x, total_boxes=total_num_spatial_boxes,
                                spatial_overlap=spatial_overlap, image_shape=shape)
                     for x in range(total_num_spatial_boxes)]
    all_rois = []
    all_boxes_eigen_vectors = []
    for spatial_box in spatial_boxes:
        spatial_box_data_list = [spatial_box.extract_box(x) for x in image_data]
        if len(spatial_box_data_list) == 1:
            spatial_box_data_list = dask.compute(*spatial_box_data_list)
        if total_num_time_steps != 1 and len(spatial_box_data_list) == 1:
            time_boxes = [(x * (image_data[0].shape[0] // total_num_time_steps), (x + 1) * (image_data[0].shape[0] //
                                                                              total_num_time_steps))
                          for x in range(total_num_time_steps)]
        all_eigen_vectors_list = []
        if not eigen_vectors_already_generated:
            for temporal_box_num, time_box_data in enumerate(
                    time_boxes if total_num_time_steps != 1 and len(
                            spatial_box_data_list) == 1 else spatial_box_data_list):
                # TODO make sure memory doesn't take more than 2x
                if total_num_time_steps != 1 and len(spatial_box_data_list) == 1:
                    time_box_data = sliceData(spatial_box_data_list,
                                              start_end=time_box_data)
                # time_box_data = saveTempImage(time_box_data, save_dir,
                #                               spatial_box.box_num)
                time_box_data_2d = reshape_to_2d_over_time(time_box_data)
                logger1.debug(
                    "Time box {0}, start {1}, end {2}, time_box shape {3}, 2d shape {4}".format(
                        temporal_box_num, 0, 0, time_box_data.shape,
                        time_box_data_2d.shape))
                k = calcAffinityMatrix(pixel_list=time_box_data_2d, metric=metric,
                                       knn=knn, accuracy=accuracy,
                                       connections=connections,
                                       normalize_w_k=normalize_w_k,
                                       num_threads=num_threads,
                                       spatial_box_num=spatial_box.box_num,
                                       temporal_box_num=temporal_box_num)
                eigen_vectors = generateEigenVectors(K=k,
                                                     num_eig=num_eig,
                                                     )
                if save_intermediate_steps:
                    eigen_vectors = saveEigenVectors(e_vectors=eigen_vectors,
                                                     spatial_box_num=spatial_box.box_num,
                                                     time_box_num=temporal_box_num,
                                                     save_dir=save_dir)

                all_eigen_vectors_list.append(eigen_vectors)
                if test_images:
                    pass
                    # delayed(save_eigen_images)(eigen_vectors=eigen_vectors,
                    #                            output_dir=test_output_dir,
                    #                            image_shape=spatial_box_data.shape,
                    #                            box_num=spatial_box.box_num).compute()

        else:
            for temporal_box_num in range(len(image_data)):
                all_eigen_vectors_list.append(
                    loadEigenVectors(spatial_box_num=spatial_box.box_num,
                                     time_box_num=temporal_box_num,
                                     save_dir=save_dir))

        all_eigen_vectors = delayed(np.hstack)(all_eigen_vectors_list)
        all_boxes_eigen_vectors.append(all_eigen_vectors)
        if save_embedding_images:
            all_eigen_vectors = saveEmbedingNormImage(e_vectors=all_eigen_vectors,
                                                      image_shape=spatial_box.shape,
                                                      save_dir=save_dir,
                                                      spatial_box_num=spatial_box.box_num)
        spatial_box_data_all = join_data_list(spatial_box_data_list)
        rois = roi_extract_image(e_vectors=all_eigen_vectors,
                                 original_shape=spatial_box_data_all.shape,
                                 original_2d_vol=reshape_to_2d_over_time(
                                     spatial_box_data_all),
                                 merge=merge,
                                 num_rois=num_rois, refinement=refinement,
                                 num_eigen_vector_select=num_eigen_vector_select,
                                 max_iter=max_iter,
                                 roi_size_min=roi_size_min,
                                 fill_holes=fill_holes,
                                 elbow_threshold_method=elbow_threshold_method,
                                 elbow_threshold_value=elbow_threshold_value,
                                 eigen_threshold_method=eigen_threshold_method,
                                 eigen_threshold_value=eigen_threshold_value,
                                 merge_temporal_coef=merge_temporal_coef,
                                 roi_size_limit=roi_size_max,
                                 box_num=spatial_box.box_num)
        if test_images:
            pass
            # delayed(save_roi_images)(
            #     roi_list=spatial_box.redefine_spatial_cord_1d(rois),
            #                              output_dir=test_output_dir,
            #                              image_shape=shape,
            #                              box_num=spatial_box.box_num).compute()
        all_rois.append(spatial_box.redefine_spatial_cord_1d(rois))
    all_rois = delayed(reduce)(lambda x, y: x + y, all_rois)
    all_rois = all_rois.compute()

    all_rois_merged = delayed(merge_rois)(roi_list=all_rois,
                                          temporal_coefficient=merge_temporal_coef,
                                          original_2d_vol=reshape_to_2d_over_time(
                                              join_data_list(image_data))).compute()

    if test_images:
        delayed(save_roi_images)(roi_list=all_rois_merged,
                                 output_dir=test_output_dir,
                                 image_shape=shape, box_num="all").compute()
    if save_embedding_images and save_intermediate_steps:
        createEmbedingNormImageFromMultiple(spatial_box_list=spatial_boxes,
                                            save_dir=save_dir,
                                            num_time_steps=len(image_data))

    return all_rois_merged


if __name__ == '__main__':
    process_data(num_threads=1, load_data=True,
                 data_path="/Users/sschickler/Code Devel/LSSC-python/input_images" +
                           "/small_dataset.tif",
                 test_images=True,
                 save_dir="/Users/sschickler/Code Devel/LSSC-python/output_images/15",
                 save_intermediate_steps=False,
                 eigen_vectors_already_generated=False,
                 save_embedding_images=False,
                 test_output_dir="/Users/sschickler/Code Devel/LSSC-python/output_images/15",
                 image_data=None,
                 total_num_time_steps=4, total_num_spatial_boxes=4, spatial_overlap=10,
                 filter=True, median_filter_size=(1, 3, 3), median_filter=True,
                 z_score=False, slice_stack=False, slice_every=10, slice_start=0,
                 metric="l2", knn=50, accuracy=59, connections=60,
                 num_eig=50, normalize_w_k=2, merge=True,
                 num_rois=25, refinement=True,
                 num_eigen_vector_select=5,
                 max_iter=400, roi_size_min=30,
                 fill_holes=True,
                 elbow_threshold_method=True,
                 elbow_threshold_value=1,
                 eigen_threshold_method=True,
                 eigen_threshold_value=.5,
                 merge_temporal_coef=.01,
                 roi_size_max=600)
    # with performance_report(filename="dask-report.html"):
    # process_data(num_threads=1, load_data=True,
    #              data_path="/Users/sschickler/Code Devel/LSSC-python/input_images/dataset_1",
    #              test_images=False,
    #              test_output_dir="/Users/sschickler/Documents/LSSC-python/output_images/16",
    #              image_data=None,
    #              save_dir="/Users/sschickler/Code DEvel/LSSC-python/output_images/16",
    #              save_embedding_images=True,
    #              save_intermediate_steps=True,
    #              eigen_vectors_already_generated=False,
    #              total_num_time_steps=20, total_num_spatial_boxes=16, spatial_overlap=30,
    #              filter=True, median_filter_size=(1, 3, 3), median_filter=True,
    #              z_score=False, slice_stack=False, slice_every=10, slice_start=0,
    #              metric="l2", knn=50, accuracy=59, connections=60,
    #              num_eig=50, normalize_w_k=2, merge=True,
    #              num_rois=25, refinement=True,
    #              num_eigen_vector_select=5,
    #              max_iter=400, roi_size_min=30,
    #              fill_holes=True,
    #              elbow_threshold_method=True,
    #              elbow_threshold_value=1,
    #              eigen_threshold_method=True,
    #              eigen_threshold_value=.5,
    #              merge_temporal_coef=.01,
    #              roi_size_max=600)


@delayed
def sliceData(data, start_end):
    return data[0][start_end[0]:start_end[1], :, :]



import json
import logging
from functools import reduce

import dask
import numpy as np
from PIL import Image
from dask import delayed
from skimage import feature

from CIDAN.LSSC.SpatialBox import SpatialBox
from CIDAN.LSSC.functions.data_manipulation import load_filter_tif_stack, filter_stack, \
    reshape_to_2d_over_time, pixel_num_to_2d_cord
from CIDAN.LSSC.functions.eigen import loadEigenVectors
from CIDAN.LSSC.functions.pickle_funcs import *
from CIDAN.LSSC.functions.roi_extraction import roi_extract_image, combine_rois
from CIDAN.LSSC.functions.roi_filter import filterRoiList
from CIDAN.LSSC.process_data import process_data
from CIDAN.TimeTrace.deltaFOverF import calculateDeltaFOverF
from CIDAN.TimeTrace.mean import calculateMeanTrace

logger1 = logging.getLogger("CIDAN.DataHandler")


class DataHandler:
    _global_params_default = {
        "save_intermediate_steps": True,
        "need_recalc_dataset_params": True,
        "need_recalc_filter_params": True,
        "need_recalc_box_params": True,
        "need_recalc_eigen_params": True,
        "need_recalc_roi_extraction_params": True,
        "num_threads": 1
    }

    _dataset_params_default = {
        "dataset_path": "",
        "trials_loaded": [],
        "trials_all": [],
        "slice_stack": False,
        "slice_every": 3,
        "slice_start": 0,
        "crop_stack": False,
        "crop_x": [0, 0],
        "crop_y": [0, 0]
    }

    _filter_params_default = {
        "median_filter": False,
        "median_filter_size": 3,
        "z_score": False

    }
    _box_params_default = {
        "total_num_time_steps": 1,
        "total_num_spatial_boxes": 1,
        "spatial_overlap": 30
    }
    _eigen_params_default = {
        "eigen_vectors_already_generated": False,
        "num_eig": 50,
        "normalize_w_k": 35,
        "metric": "l2",
        "knn": 45,
        "accuracy": 50,
        "connections": 51,

    }
    _roi_extraction_params_default = {
        "elbow_threshold_method": True,
        "elbow_threshold_value": 1,
        "eigen_threshold_method": True,
        "eigen_threshold_value": .1,
        "num_eigen_vector_select": 2,
        "merge_temporal_coef": .95,
        "roi_size_min": 30,
        "roi_size_max": 600,
        "merge": True,
        "num_rois": 60,
        "fill_holes": True,
        "refinement": True,
        "max_iter": 40,
        "roi_circ_threshold": .9
    }
    _time_trace_params_default = {
        "time_trace_type": "Mean"
    }

    def __init__(self, data_path, save_dir_path, save_dir_already_created, trials=[]):
        # TODO add loaded trials and all trials parameter here
        # TODO make sure if trial list includes files that aren't valid it works
        self.color_list = [(218, 67, 34),
                           (132, 249, 22), (22, 249, 140), (22, 245, 249),
                           (22, 132, 249), (224, 22, 249), (249, 22, 160)]

        self.save_dir_path = save_dir_path
        self.rois_loaded = False
        if save_dir_already_created:
            valid = self.load_param_json()
            self._trials_loaded_indices = [num for num, x in enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]
            self.calculate_filters()

            if self.rois_exist:
                self.load_rois()
                self.calculate_time_traces()
                self.rois_loaded = True
            if not valid:
                raise FileNotFoundError("Save directory not valid")
        else:
            self.global_params = DataHandler._global_params_default.copy()

            self.dataset_params = DataHandler._dataset_params_default.copy()
            self.dataset_params["dataset_path"] = data_path
            self.dataset_params["trials_loaded"] = trials
            self.trials_loaded = trials
            if (data_path == ""):
                self.trials_all = trials
            else:
                self.trials_all = sorted(os.listdir(data_path))
                if not os.path.isdir(os.path.join(data_path, trials[0])):
                    self.trials_all = [x for x in self.trials_all if ".tif" in x]
            self.dataset_params["trials_all"] = self.trials_all

            self.filter_params = DataHandler._filter_params_default.copy()
            self.box_params = DataHandler._box_params_default.copy()
            self.box_params_processed = DataHandler._box_params_default.copy()
            self.box_params["total_num_time_steps"] = len(trials)
            self.eigen_params = DataHandler._eigen_params_default.copy()
            self.roi_extraction_params = DataHandler._roi_extraction_params_default.copy()
            self.time_trace_params = DataHandler._time_trace_params_default.copy()
            valid = self.create_new_save_dir()
            if not valid:
                raise FileNotFoundError("Please chose an empty directory for your " +
                                        "save directory")
            self.time_traces = []
            self._trials_loaded_indices = [num for num, x in enumerate(self.trials_all)
                                           if x in self.trials_loaded]
            self.trials_loaded_time_trace_indices = [num for num, x in
                                                     enumerate(self.trials_all)
                                                     if x in self.trials_loaded]

    def __del__(self):
        try:
            for x in self.__dict__.items():
                self.__dict__[x] = None
        except TypeError:
            pass

    @property
    def dataset_trials_filtered_loaded(self):
        return [self.dataset_trials_filtered[x] for x in self._trials_loaded_indices]

    @property
    def dataset_trials_loaded(self):
        return [self.dataset_trials[x] for x in self._trials_loaded_indices]

    @property
    def param_path(self):
        return os.path.join(self.save_dir_path, "parameters.json")

    @property
    def eigen_vectors_exist(self):
        eigen_dir = os.path.join(self.save_dir_path, "eigen_vectors")

        file_names = [
            "eigen_vectors_box_{}_{}.pickle".format(spatial_box_num, time_box_num)
            for spatial_box_num in range(self.box_params["total_num_spatial_boxes"])
            for time_box_num in range(self.box_params["total_num_time_steps"])]
        return all(pickle_exist(x, output_directory=eigen_dir) for x in file_names)

    @property
    def rois_exist(self):
        return pickle_exist("rois", output_directory=self.save_dir_path)

    def load_rois(self):
        if pickle_exist("rois", output_directory=self.save_dir_path):
            self.clusters = pickle_load("rois", output_directory=self.save_dir_path)
            self.gen_roi_display_variables()

    def save_rois(self, rois):
        if os.path.isdir(self.save_dir_path):
            pickle_save(rois, "rois", output_directory=self.save_dir_path)

    def load_param_json(self):

        try:
            with open(self.param_path, "r") as f:
                all_params = json.loads(f.read())
            self.global_params = all_params["global_params"]
            self.dataset_params = all_params["dataset_params"]
            self.filter_params = all_params["filter_params"]
            self.box_params = all_params["box_params"]
            self.box_params_processed = all_params["box_params"].copy()
            self.eigen_params = all_params["eigen_params"]
            self.time_trace_params = all_params["time_trace_params"]
            self.roi_extraction_params = all_params["roi_extraction_params"]
            self.trials_loaded = self.dataset_params["trials_loaded"]
            self.trials_all = self.dataset_params["trials_all"]
            return True
        except KeyError:
            raise KeyError("Please Choose a valid parameter file")
        except FileNotFoundError:
            raise FileNotFoundError("Can't find parameter file")
        except NameError:
            raise FileNotFoundError("Can't find parameter file")

    def save_new_param_json(self):
        try:
            with open(self.param_path, "w") as f:
                all_params = {
                    "global_params": self.global_params,
                    "dataset_params": self.dataset_params,
                    "filter_params": self.filter_params,
                    "box_params": self.box_params_processed,
                    "eigen_params": self.eigen_params,
                    "roi_extraction_params": self.roi_extraction_params,
                    "time_trace_params": self.time_trace_params
                }
                f.truncate(0)
                f.write(json.dumps(all_params))
        except:
            raise FileNotFoundError("Error saving parameters, please restart software")
        pass

    def create_new_save_dir(self):
        try:
            if not os.path.isdir(self.save_dir_path):
                os.mkdir(self.save_dir_path)
            eigen_vectors_folder_path = os.path.join(self.save_dir_path,
                                                     "eigen_vectors/")
            if not os.path.isdir(eigen_vectors_folder_path):
                os.mkdir(eigen_vectors_folder_path)
            embedding_images_path = os.path.join(self.save_dir_path,
                                                 "embedding_norm_images/")
            if not os.path.isdir(embedding_images_path):
                os.mkdir(embedding_images_path)
            return True
        except:
            raise FileNotFoundError("Couldn't create folder please try again")

    def change_global_param(self, param_name, new_value):
        if param_name in self.global_params:
            self.global_params[param_name] = new_value
            self.save_new_param_json()
            return True
        else:
            return False

    def change_dataset_param(self, param_name, new_value):
        if param_name in self.dataset_params:
            self.dataset_params[param_name] = new_value
            self.global_params["need_recalc_dataset_params"] = True
            self.save_new_param_json()
            return True
        else:
            return False

    def change_filter_param(self, param_name, new_value):
        if param_name in self.filter_params:
            self.filter_params[param_name] = new_value
            self.global_params["need_recalc_filter_params"] = True
            self.save_new_param_json()
            return True
        else:
            return False

    def change_box_param(self, param_name, new_value):
        if param_name in self.box_params:
            # if param_name == "total_num_spatial_boxes":
            #     assert (int(new_value**.5))**2 == new_value, "Please make sure Number of Spatial Boxes is a square number"
            self.box_params[param_name] = new_value
            # self.global_params["need_recalc_box_params"] = True
            self.global_params["need_recalc_eigen_params"] = True
            # self.global_params["need_recalc_roi_extraction_params"] = True
            self.save_new_param_json()
            return True
        else:
            return False

    def change_eigen_param(self, param_name, new_value):
        if param_name in self.eigen_params:
            self.eigen_params[param_name] = new_value
            self.global_params["need_recalc_eigen_params"] = True
            self.save_new_param_json()
            return True
        else:
            return False

    def change_roi_extraction_param(self, param_name, new_value):
        if param_name in self.roi_extraction_params:
            self.roi_extraction_params[param_name] = new_value
            self.global_params["need_recalc_roi_extraction_params"] = True
            self.save_new_param_json()
            return True
        else:
            return False

    def calculate_dataset(self) -> np.ndarray:
        """Loads the dataset

        Returns
        -------
        """
        # TODO make it so it does't load the dataset every time

        self.dataset_trials = [False] * len(self.trials_all)
        for trial_num in self._trials_loaded_indices:
            self.dataset_trials[trial_num] = self.load_trial_dataset_step(trial_num)
        self.dataset_trials = dask.compute(*self.dataset_trials)
        self.global_params["need_recalc_dataset_params"] = False
        self.shape = [self.dataset_trials_loaded[0].shape[1],
                      self.dataset_trials_loaded[0].shape[2]]
        if self.dataset_params["crop_x"][1] == 0:
            self.dataset_params["crop_x"][1] = self.shape[0]
            self.dataset_params["crop_y"][1] = self.shape[1]
        print("Finished Calculating Dataset")

    @delayed
    def load_trial_dataset_step(self, trial_num):
        return load_filter_tif_stack(
            path=os.path.join(self.dataset_params["dataset_path"],
                              self.trials_all[trial_num]),
            filter=False,
            median_filter=False,
            median_filter_size=(1, 3, 3),
            z_score=False,
            slice_stack=self.dataset_params[
                "slice_stack"],
            slice_start=self.dataset_params[
                "slice_start"],

            slice_every=self.dataset_params[
                "slice_every"],
            crop_stack=self.dataset_params[
                "crop_stack"],
            crop_x=self.dataset_params[
                "crop_x"],
            crop_y=self.dataset_params[
                "crop_y"]
        )

    @delayed
    def load_trial_filter_step(self, trial_num):

        return filter_stack(
            stack=self.dataset_trials[trial_num] if type(self.dataset_trials[
                                                             trial_num]) != bool else self.load_trial_dataset_step(
                trial_num).compute(),
            median_filter_size=(self.filter_params[
                                    "median_filter_size"],
                                self.filter_params[
                                    "median_filter_size"],
                                self.filter_params[
                                    "median_filter_size"]),
            median_filter=self.filter_params[
                "median_filter"],
            z_score=self.filter_params["z_score"])

    def calculate_filters(self):

        if self.global_params["need_recalc_filter_params"] or self.global_params[
            "need_recalc_dataset_params"] or \
                not hasattr(self, "dataset_trials_filtered"):
            self.calculate_dataset()
            self.dataset_trials_filtered = [False] * len(self.trials_all)
            for trial_num in self._trials_loaded_indices:
                self.dataset_trials_filtered[trial_num] = self.load_trial_filter_step(
                    trial_num)
            self.dataset_trials_filtered = dask.compute(*self.dataset_trials_filtered)
            self.mean_image = np.mean(np.dstack(
                [np.mean(x, axis=0) for x in self.dataset_trials_filtered_loaded]),
                axis=2)
            self.max_image = np.max(np.dstack(
                [np.max(x, axis=0) for x in self.dataset_trials_filtered_loaded]),
                axis=2)

            # self.temporal_correlation_image = calculate_temporal_correlation(self.dataset_filtered)
            self.global_params["need_recalc_filter_params"] = False
            self.global_params["need_recalc_box_params"] = True
        return self.dataset_trials_filtered

    def calculate_roi_extraction(self):
        if self.global_params["need_recalc_eigen_params"] or self.global_params[
            "need_recalc_roi_extraction_params"] or self.global_params[
            "need_recalc_box_parmas"] or self.global_params[
            "need_recalc_dataset_params"] or \
                self.global_params["need_recalc_filter_params"]:
            assert (int(
                self.box_params[
                    "total_num_spatial_boxes"] ** .5)) ** 2 == self.box_params[
                       "total_num_spatial_boxes"], "Please make sure Number of Spatial Boxes is a square number"
            try:
                self.calculate_filters()
                eigen_need_recalc = self.global_params["need_recalc_eigen_params"]
                self.global_params["need_recalc_eigen_params"] = False
                self.clusters = process_data(num_threads=self.global_params[
                    "num_threads"], test_images=False, test_output_dir="",
                                             save_dir=self.save_dir_path,
                                             save_intermediate_steps=self.global_params[
                                                 "save_intermediate_steps"],
                                             image_data=self.dataset_trials_filtered_loaded,
                                             eigen_vectors_already_generated=(not
                                                                              eigen_need_recalc) and
                                                                             self.global_params[
                                                                                 "save_intermediate_steps"] and self.eigen_vectors_exist,
                                             save_embedding_images=True,
                                             total_num_time_steps=self.box_params[
                                                 "total_num_time_steps"],
                                             total_num_spatial_boxes=self.box_params[
                                                 "total_num_spatial_boxes"],
                                             spatial_overlap=self.box_params[
                                                                 "spatial_overlap"] // 2,

                                             metric=self.eigen_params["metric"],
                                             knn=self.eigen_params["knn"],
                                             accuracy=self.eigen_params["accuracy"],
                                             connections=self.eigen_params[
                                                 "connections"],
                                             normalize_w_k=self.eigen_params[
                                                 "normalize_w_k"],
                                             num_eig=self.eigen_params["num_eig"],
                                             merge=self.roi_extraction_params["merge"],
                                             num_rois=self.roi_extraction_params[
                                                 "num_rois"],
                                             refinement=self.roi_extraction_params[
                                                 "refinement"],
                                             num_eigen_vector_select=
                                             self.roi_extraction_params[
                                                 "num_eigen_vector_select"],
                                             max_iter=self.roi_extraction_params[
                                                 "max_iter"],
                                             roi_size_min=self.roi_extraction_params[
                                                 "roi_size_min"],
                                             fill_holes=self.roi_extraction_params[
                                                 "fill_holes"],

                                             elbow_threshold_method=
                                             self.roi_extraction_params[
                                                 "elbow_threshold_method"],
                                             elbow_threshold_value=
                                             self.roi_extraction_params[
                                                 "elbow_threshold_value"],
                                             eigen_threshold_method=
                                             self.roi_extraction_params[
                                                 "eigen_threshold_method"],
                                             eigen_threshold_value=
                                             self.roi_extraction_params[
                                                 "eigen_threshold_value"],
                                             merge_temporal_coef=
                                             self.roi_extraction_params[
                                                 "merge_temporal_coef"],
                                             roi_size_max=self.roi_extraction_params[
                                                 "roi_size_max"])
                self.box_params_processed = self.box_params
                self.save_new_param_json()
                roi_valid_list = filterRoiList(self.clusters, self.shape)
                self.clusters = [x for x, y in zip(self.clusters, roi_valid_list) if
                                 y > self.roi_extraction_params[
                                     "roi_circ_threshold"]]
                self.save_rois(self.clusters)
                print("Calculating Time Traces:")
                self.gen_roi_display_variables()
                self.calculate_time_traces()

                self.rois_loaded = True
            except FileNotFoundError as e:
                self.global_params["need_recalc_eigen_params"] = False
                logger1.error(e)
                print(
                    "Please try again there was an internal error in the roi extraction process")
                raise AssertionError()

    def gen_roi_display_variables(self):

        self.roi_circ_list = filterRoiList(self.clusters, self.shape,
                                           self.roi_extraction_params[
                                               "roi_circ_threshold"])
        cluster_list_2d_cord = [
            pixel_num_to_2d_cord(x, volume_shape=self.shape) for x in
            self.clusters]
        self.cluster_max_cord_list = [np.max(x, axis=1) for x in cluster_list_2d_cord]
        self.cluster_min_cord_list = [np.min(x, axis=1) for x in cluster_list_2d_cord]
        self.pixel_with_rois_flat = np.zeros(
            [self.shape[0] * self.shape[1]])
        self.pixel_with_rois_color_flat = np.zeros(
            [self.shape[0] * self.shape[1], 3])
        for num, cluster in enumerate(self.clusters):
            cur_color = self.color_list[num % len(self.color_list)]
            self.pixel_with_rois_flat[cluster] = num + 1
            self.pixel_with_rois_color_flat[cluster] = cur_color
        edge_roi_image = feature.canny(np.reshape(self.pixel_with_rois_flat,
                                                  [self.shape[0],
                                                   self.shape[1]]))
        self.edge_roi_image_flat = np.reshape(edge_roi_image, [-1, 1]) * 255
        self.pixel_with_rois_color = np.reshape(self.pixel_with_rois_color_flat,
                                                [self.shape[0],
                                                 self.shape[1], 3])
        try:
            self.eigen_norm_image = np.asarray(Image.open(
                os.path.join(self.save_dir_path,
                             "embedding_norm_images/embedding_norm_image.png")))
        except:
            print("Can't generate eigen Norm image please try again")

    def calculate_time_traces(self):
        self.time_traces = []
        for _ in range(len(self.clusters)):
            self.time_traces.append([])
            for _ in range(len(self.trials_all)):
                self.time_traces[-1].append(False)
        for trial_num in self.trials_loaded_time_trace_indices:
            for cluster in range(len(self.clusters)):
                self.calculate_time_trace(cluster + 1, trial_num)

        if os.path.isdir(self.save_dir_path):
            pickle_save(self.time_traces, "time_traces",
                        output_directory=self.save_dir_path)
        self.rois_loaded = True

    def calculate_time_trace(self, roi_num, trial_num=None):
        """
        Calculates a time trace for a certain ROI and save to time trace list
        Parameters
        ----------
        roi_num
            roi to calculate for this starts at [1..number of rois]
        trial_num
            indice of trial in trials_all starts at [0, number of trials-1] if
            none then calculates for all trials

        Returns
        -------

        """
        # TODO make so it can also calculate for a specific trial, so it would be a 2d array
        trial_nums = [trial_num]
        if trial_num == None:
            trial_nums = self.trials_loaded_time_trace_indices
        for trial_num in trial_nums:
            cluster = self.clusters[roi_num - 1]
            if type(self.dataset_trials_filtered[trial_num]) == bool:
                self.load_trial_filter_step(trial_num)
            data_2d = reshape_to_2d_over_time(self.dataset_trials_filtered[trial_num])
            if self.time_trace_params["time_trace_type"] == "DeltaF/F":
                time_trace = calculateDeltaFOverF(cluster, data_2d)
            else:
                time_trace = calculateMeanTrace(cluster, data_2d)
            calculateDeltaFOverF(cluster, data_2d)
            self.time_traces[roi_num - 1][trial_num] = time_trace
        if os.path.isdir(self.save_dir_path):
            pickle_save(self.time_traces, "time_traces",
                        output_directory=self.save_dir_path)

    def get_time_trace(self, num):
        num = num - 1
        output = np.ndarray([])
        for trial_num in self.trials_loaded_time_trace_indices:
            if type(self.time_traces[num][trial_num]) == bool:
                self.calculate_time_trace(num + 1, trial_num)
            output = np.hstack([output, self.time_traces[num][trial_num]])

        return output

    def update_selected_trials(self, selected_trials):
        """
        Updates the loaded trials so don't have unnecessary loaded trials
        Parameters
        ----------
        selected_trials
            trials to be loaded

        Returns
        -------

        """
        new_selected = [x for x, y in enumerate(self.trials_all) if
                        y in selected_trials]
        for trial_num, _ in enumerate(self.trials_all):
            if trial_num in new_selected and trial_num not in self.trials_loaded_time_trace_indices:
                self.dataset_trials_filtered = self.load_trial_filter_step(
                    trial_num).compute()
                self.trials_loaded_time_trace_indices.append(trial_num)
            if trial_num not in new_selected and trial_num in self.trials_loaded_time_trace_indices \
                    and trial_num not in self._trials_loaded_indices:
                self.dataset_trials_filtered[trial_num] = False

            if trial_num not in new_selected and trial_num in self.trials_loaded_time_trace_indices:
                self.trials_loaded_time_trace_indices.remove(trial_num)

    def get_eigen_vector(self, box_num, vector_num, trial_num):
        vector = loadEigenVectors(spatial_box_num=box_num, time_box_num=trial_num,
                                  save_dir=self.save_dir_path).compute()
        spatial_box = SpatialBox(box_num,
                                 total_boxes=self.box_params["total_num_spatial_boxes"],
                                 image_shape=self.shape,
                                 spatial_overlap=self.box_params[
                                                     "spatial_overlap"] // 2)
        return vector[:, vector_num].reshape(spatial_box.shape)

    def genRoiFromPoint(self, point, growth_factor=1):
        """
        Generates an roi from an initial point in the image
        Parameters
        ----------
        point 2d cordinates of a point in the image
        growth_factor how much to grow the roi

        Returns
        -------
        the pixels in said roi
        """
        spatial_boxes = [SpatialBox(box_num=x,
                                    total_boxes=self.box_params_processed[
                                        "total_num_spatial_boxes"],
                                    image_shape=self.shape,
                                    spatial_overlap=self.box_params_processed[
                                        "spatial_overlap"]) for x in
                         range(self.box_params_processed["total_num_spatial_boxes"])]
        boxes_point_in = [x for x in spatial_boxes if x.pointInBox(point)]

        rois = []
        for box in boxes_point_in:
            all_eigen_vectors_list = []
            for temporal_box_num in range(
                    self.box_params_processed["total_num_time_steps"]):
                all_eigen_vectors_list.append(
                    loadEigenVectors(spatial_box_num=box.box_num,
                                     time_box_num=temporal_box_num,
                                     save_dir=self.save_dir_path).compute())

            all_eigen_vectors = np.hstack(all_eigen_vectors_list)
            box_point = box.point_to_box_point(point)
            box_point_1d = box_point[0] * box.shape[1] + box_point[1]
            roi = roi_extract_image(e_vectors=all_eigen_vectors,
                                    original_shape=box.shape, original_2d_vol=None,
                                    merge=False, box_num=box.box_num,
                                    num_rois=1, refinement=True,
                                    num_eigen_vector_select=
                                    self.roi_extraction_params[
                                        "num_eigen_vector_select"],
                                    max_iter=1,
                                    roi_size_min=0,
                                    fill_holes=self.roi_extraction_params[
                                        "fill_holes"],

                                    elbow_threshold_method=True,
                                    elbow_threshold_value=growth_factor,
                                    eigen_threshold_method=
                                    self.roi_extraction_params[
                                        "eigen_threshold_method"],
                                    eigen_threshold_value=
                                    self.roi_extraction_params[
                                        "eigen_threshold_value"],
                                    merge_temporal_coef=
                                    self.roi_extraction_params[
                                        "merge_temporal_coef"],
                                    roi_size_limit=1000,
                                    initial_pixel=box_point_1d,
                                    print_info=False).compute()

            if len(roi) > 0 and box_point_1d in roi[0]:
                rois.append(box.redefine_spatial_cord_1d(roi).compute()[0])
        if len(rois) > 0:
            final_roi = reduce(combine_rois, rois)
        else:
            return []

        return final_roi



import logging
from typing import Tuple

import numpy as np
from dask import delayed

logger1 = logging.getLogger("CIDAN.LSSC.SpatialBox")


class SpatialBox:
    def __init__(self, box_num: int, total_boxes: int, image_shape: Tuple[int, int],
                 spatial_overlap: int):
        logger1.debug(
            "Spatial Box creation inputs: box num {0}, total boxes {1}, image shape {2}, spatial overlap {3}".format(
                box_num, total_boxes, image_shape, spatial_overlap))
        # TODO implement spatial overlap
        self.box_num = box_num
        self.total_boxes = total_boxes
        self.image_shape = image_shape
        self.boxes_per_row = int(total_boxes ** .5)
        self.spatial_overlap = spatial_overlap
        self.y_box_num = box_num // self.boxes_per_row
        self.x_box_num = box_num - (self.y_box_num * self.boxes_per_row)

        self.box_cord_1 = [((image_shape[0] // self.boxes_per_row) *
                            self.x_box_num) - spatial_overlap,
                           (image_shape[1] // self.boxes_per_row) *
                           self.y_box_num - spatial_overlap]
        self.box_cord_2 = [(image_shape[0] // self.boxes_per_row) * (
                self.x_box_num + 1) + spatial_overlap,
                           (image_shape[1] // self.boxes_per_row) * (
                                   self.y_box_num + 1) + spatial_overlap]

        self.box_cord_1[0] = 0 if self.box_cord_1[0] < 0 else self.box_cord_1[0]
        self.box_cord_1[1] = 0 if self.box_cord_1[1] < 0 \
            else self.box_cord_1[1]
        self.box_cord_2[0] = image_shape[0] if self.box_cord_2[0] > image_shape[0] \
            else \
            self.box_cord_2[0]
        self.box_cord_2[1] = image_shape[1] if self.box_cord_2[1] > image_shape[1] \
            else self.box_cord_2[1]
        self.shape = (self.box_cord_2[0] - self.box_cord_1[0],
                      self.box_cord_2[
                          1] - self.box_cord_1[1])
        logger1.debug(("Spatial box creation: Boxes per row {0}, y_box_num {1}, " +
                       "x_box_num" + " {2}, box cord 1 {3}, box cord 2 {4}, shape {5}"
                       ).format(self.boxes_per_row, self.y_box_num, self.x_box_num,
                                self.box_cord_1, self.box_cord_2, self.shape))

    @delayed
    def extract_box(self, dataset):
        return dataset[:, self.box_cord_1[0]:self.box_cord_2[0], self.box_cord_1[1]:
                                                                 self.box_cord_2[1]]

    @delayed
    def redefine_spatial_cord_2d(self, cord_list):
        return [(x + self.box_cord_1[0], y + self.box_cord_1[1]) for x, y in cord_list]

    def pointInBox(self, point):
        """
        Checks if a point is in the box
        Parameters
        ----------
        point

        Returns
        -------

        """
        return self.box_cord_1[0] <= point[0] <= self.box_cord_2[0] \
               and self.box_cord_1[1] <= point[1] <= self.box_cord_2[1]

    def point_to_box_point(self, point):
        """
        Converts a point in the image to its cords in the box
        Parameters
        ----------
        point 2d point

        Returns
        -------
        tuple of new cords
        """
        return (point[0] - self.box_cord_1[0], point[1] - self.box_cord_1[1])
    @delayed
    def redefine_spatial_cord_1d(self, cord_list):
        box_length = self.box_cord_2[1] - self.box_cord_1[1]

        def change_1_cord(x):
            return ((x // box_length) + self.box_cord_1[0]) * self.image_shape[
                1] + self.box_cord_1[1] + x % box_length

        return list(map(change_1_cord, cord_list))

    def convert_1d_to_2d(self):
        # TODO implement this
        pass

    def data_w_out_spatial_overlap(self, data):
        """

        Parameters
        ----------
        data 2d dataset

        Returns
        -------

        """
        if self.total_boxes == 1:
            return data
        x = [0, self.shape[0]]
        y = [0, self.shape[1]]
        # This uses which column each box is in to determin overlap parts, first, last,
        # any other column
        if self.box_num % self.boxes_per_row == 0:
            x[1] = self.shape[0] - self.spatial_overlap
        elif self.box_num % self.boxes_per_row == self.boxes_per_row - 1:
            x[0] = self.spatial_overlap
        else:
            x[0] = self.spatial_overlap
            x[1] = self.shape[0] - self.spatial_overlap
        # This uses which row each box is in to determin overlap parts, first, last,
        # any other row
        if self.box_num // self.boxes_per_row == 0:
            y[1] = self.shape[1] - self.spatial_overlap
        elif self.box_num // self.boxes_per_row == self.boxes_per_row - 1:
            y[0] = self.spatial_overlap
        else:
            y[0] = self.spatial_overlap
            y[1] = self.shape[1] - self.spatial_overlap
        return data[x[0]:x[1], y[0]:y[1]]


def combine_images(spatial_box_list, data_list):
    """

    Parameters
    ----------
    spatial_box_list
    data_list list of reshaped eigen vectors in the correct shape for these boxes

    Returns
    -------

    """
    # Going to go through the lists in a 2d format using number of spatial boxes is always square
    spatial_box_num = len(spatial_box_list)
    spatial_box_root = int(spatial_box_num ** .5)
    data_matched = []
    for y in range(spatial_box_root):
        temp = []
        for x in range(spatial_box_root):
            current_spatial_box = spatial_box_list[y * spatial_box_root + x]
            current_data = data_list[y * spatial_box_root + x]
            temp.append(current_spatial_box.data_w_out_spatial_overlap(current_data))
        stacked = np.vstack(temp)
        data_matched.append(stacked)
    all_data = np.hstack(data_matched)

    return all_data


if __name__ == '__main__':
    test = SpatialBox(box_num=0, total_boxes=9, image_shape=[1, 9, 9],
                      spatial_overlap=0)
    pixel_list = test.redefine_spatial_cord_1d([0, 4, 8]).compute()

    zeros = np.zeros((9 * 9))
    zeros[pixel_list] = 1
    print(zeros.reshape((9, 9)))



import logging
import os

import numpy as np
from PIL import Image
from dask import delayed
from scipy import sparse
from scipy.sparse import linalg

from CIDAN.LSSC.SpatialBox import combine_images
from CIDAN.LSSC.functions.embeddings import calcDSqrt, calcDNegSqrt, calcDInv
from CIDAN.LSSC.functions.pickle_funcs import pickle_save, pickle_load

logger1 = logging.getLogger("CIDAN.LSSC.eigen")


@delayed
def generateEigenVectors(*, K: sparse.csr_matrix, num_eig: int) -> np.ndarray:
    """Calculate Eigen Vectors given parts of the affinity matrix

    Parameters
    ----------

    K
        An affinity matrix K for the image
    num_eig
        number of eigen values to generate
    Returns
    -------
    A matrix of eigen vectors
    """
    D_inv, D_diag = calcDInv(K=K)
    P = D_inv.dot(K)
    D_neg_sqrt = calcDNegSqrt(D_diag)
    P_transformed = calcDSqrt(D_diag).dot(P).dot(D_neg_sqrt)
    eig_values, eig_vectors_scaled = linalg.eigsh(
        P_transformed, num_eig, which="LM",
        return_eigenvectors=True)  # this returns normalized eigen vectors
    # TODO make first eigen vector be sanity check since all elements are the same
    #  this isn't the case
    # print("Eigvalues",eig_values[0], eig_vectors_scaled,np.max(eig_vectors_scaled),eig_vectors_scaled.shape, num_eig)
    eig_vectors = np.flip(
        D_neg_sqrt.dot(eig_vectors_scaled),
        axis=1)  # this preforms matrix multiplication

    return eig_vectors


@delayed
def saveEigenVectors(*, e_vectors, spatial_box_num: int, time_box_num: int, save_dir:
str):
    eigen_dir = os.path.join(save_dir, "eigen_vectors")
    if not os.path.isdir(eigen_dir):
        os.mkdir(eigen_dir)
    pickle_save(e_vectors, name="eigen_vectors_box_{}_{}.pickle".format(spatial_box_num,
                                                                        time_box_num),
                output_directory=eigen_dir, )
    return e_vectors


@delayed
def loadEigenVectors(*, spatial_box_num: int, time_box_num: int, save_dir: str):
    eigen_dir = os.path.join(save_dir, "eigen_vectors")
    name = "eigen_vectors_box_{}_{}.pickle".format(spatial_box_num, time_box_num)
    logger1.debug(
        "Eigen vector load: file name: {0}, directory: {1}".format(name, eigen_dir))
    vector = pickle_load(name=name,
                         output_directory=eigen_dir)
    logger1.debug(
        "Eigen spatial box {0}, time box {1} shape: {2}".format(spatial_box_num,
                                                                time_box_num,
                                                                vector.shape))  # noqa
    return vector


@delayed
def saveEmbedingNormImage(*, e_vectors, image_shape, save_dir, spatial_box_num):
    print(save_dir)
    embed_dir = os.path.join(save_dir, "embedding_norm_images")
    e_vectors_squared = np.power(e_vectors, 2)
    e_vectors_sum = np.sum(e_vectors_squared, axis=1)

    e_vectors_sum_rescaled = e_vectors_sum * (
            10.0 / e_vectors_sum.max())  # add histogram equalization

    img = Image.fromarray(
        np.reshape(e_vectors_sum_rescaled,
                   image_shape) * 255).convert('L')
    image_path = os.path.join(embed_dir, "embedding_norm_image_box_{}.png".format(
        spatial_box_num))
    img.save(image_path)
    return e_vectors


def createEmbedingNormImageFromMultiple(*, spatial_box_list, save_dir, num_time_steps):
    """
    This function takes in a place where eigen vectors are stored and creates a full
    image for the entire image instead of just each spatial box
    Parameters
    ----------
    spatial_box_list list of spatial boxes
    save_dir where eigen vectors are stored
    num_time_steps number of timesteps used

    Returns
    -------

    """
    eigen_dir = os.path.join(save_dir, "eigen_vectors")
    e_vectors_list = []
    for spatial_box in spatial_box_list:
        temp = []
        for time_box_num in range(num_time_steps):
            temp.append(pickle_load(
                "eigen_vectors_box_{}_{}.pickle".format(spatial_box.box_num,
                                                        time_box_num),
                output_directory=eigen_dir))
        e_vectors_list.append(np.hstack(temp))
    embed_dir = os.path.join(save_dir, "embedding_norm_images")
    eigen_images = []
    for e_vectors, spatial_box in zip(e_vectors_list, spatial_box_list):
        e_vectors_squared = np.power(e_vectors, 2)
        e_vectors_sum = np.sum(e_vectors_squared, axis=1)
        e_vectors_sum_rescaled = e_vectors_sum * (
                9.0 / e_vectors_sum.max())  # add histogram equalization
        # noqa
        e_vectors_shaped = np.reshape(e_vectors_sum,
                                      spatial_box.shape)
        eigen_images.append(e_vectors_shaped)
    image = combine_images(spatial_box_list, eigen_images)

    img = Image.fromarray(image * (
            3.0 / image.max()) * 255).convert('L')
    image_path = os.path.join(embed_dir, "embedding_norm_image.png")
    img.save(image_path)
    return e_vectors_list



import dask
import numpy as np
from skimage import feature

from CIDAN.LSSC.functions.data_manipulation import pixel_num_to_2d_cord


def filterRoiList(roi_list, image_shape, threshold=.7):
    """
    Filters a list of ROIS based on looking at surface area vs surface area of a similar sized circle of each roi
    Parameters
    ----------
    roi_list
    image_shape
    threshold

    Returns
    -------
    list of threshold_values
    """
    out_list = [filterRoi(x, image_shape=image_shape, threshold=threshold) for x in
                roi_list]
    return dask.compute(*out_list)


def filterRoi(roi, image_shape, threshold):
    roi_2d_cord = pixel_num_to_2d_cord(roi, volume_shape=image_shape)
    roi_max_cord = np.max(roi_2d_cord, axis=1)
    roi_min_cord = np.min(roi_2d_cord, axis=1)
    newShape = roi_max_cord - roi_min_cord + np.array([10, 10])
    image = np.zeros(newShape)
    for cord in zip(
            *list(roi_2d_cord - np.reshape(roi_min_cord - np.array([2, 2]), (2, 1)))):
        image[cord[0], cord[1]] = 1
    image_canny = feature.canny(image)
    surface_pixels = np.count_nonzero(image_canny)
    return ((((np.min(roi_max_cord - roi_min_cord)) * 3.14))) / surface_pixels



import os
import pickle

dir_name = "/data2/Sam/pythonTestEnviroment/tmp_pickle"


def pickle_set_dir(dir_path):
    global dir_name
    dir_name = dir_path


def pickle_save(obj, name, trial_num=0, output_directory=None):
    """
    saves an object as a pickle file in a folder specific to the trial
    Parameters
    ----------
    obj: object to pickle
    name: name of file
    trial_num: which trial this is part of
    output_directory: optional if want to save in specific directory
    Returns
    -------
    None
    """

    if ".pickle" not in name:
        name = name + ".pickle"

    if output_directory:
        dir_full_path = output_directory
    else:
        dir_full_path = os.path.join(dir_name, str(trial_num) + "/")
        if not os.path.isdir(dir_name):
            os.mkdir(dir_name)
    if not os.path.isdir(dir_full_path):
        os.mkdir(dir_full_path)
    with open(os.path.join(dir_full_path, name), "wb") as file:
        pickle.dump(obj, file, protocol=4)


def pickle_exist(name, trial_num=0, output_directory=None):
    """
    checks if a pickle file exists
    Parameters
    ----------
    name: name of file
    trial_num: which trial this is part of

    Returns
    -------
    None
    """
    if not output_directory:
        dir_full_path = os.path.join(dir_name, str(trial_num) + "/")
    else:
        dir_full_path = output_directory
    if ".pickle" not in name:
        name = name + ".pickle"
    return os.path.isfile(os.path.join(dir_full_path, name))


def pickle_load(name, trial_num=0, output_directory=None):
    """
    loads a pickled object
    Parameters
    ----------
    name name of file
    trial_num which trial this is part of

    Returns
    -------
    object
    """
    if not output_directory:
        dir_full_path = os.path.join(dir_name, str(trial_num) + "/")
    else:
        dir_full_path = output_directory
    if ".pickle" not in name:
        name = name + ".pickle"
    with open(os.path.join(dir_full_path, name), "rb") as file:
        return pickle.load(file)
    raise FileNotFoundError


def pickle_clear(trial_num=0):
    """
    Clears all pickle files saved in temp
    Parameters
    ----------
    trial_num which trial this is part of
    Returns
    -------
    None
    """
    try:
        dir_full_path = os.path.join(dir_name, str(trial_num) + "/")
        filelist = [f for f in os.listdir(dir_full_path) if
                    f.endswith(".pickle")]
        for f in filelist:
            os.remove(os.path.join(dir_full_path, f))
    except FileNotFoundError:
        pass



import sys

import qdarkstyle

from CIDAN.GUI.SettingWidget.SettingBlockModule import *


class SettingsModule(QFrame):
    def __init__(self, name, *args, show_name=True):
        super().__init__()

        self.setting_block_list = args
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(2, 2, 2, 2)
        if show_name:
            # self.setStyleSheet("SettingsModule { border:1px solid rgb(50, 65, "
            #                    "75);} ")
            self.header = QLabel()
            self.header.setText(name)
            self.header.setStyleSheet("font-size: 20px")
            self.layout.addWidget(self.header)

        self.setting_block_layout = QToolBox()
        for block in self.setting_block_list:
            self.setting_block_layout.addItem(block, block.name)
        self.layout.addWidget(self.setting_block_layout)
        self.setLayout(self.layout)


def preprocessing_settings(main_widget):
    return SettingsModule("Preprocessing Settings",
                          filter_setting_block(main_widget),
                          dataset_setting_block_crop(main_widget),
                          dataset_setting_block(main_widget))


def roi_extraction_settings(main_widget):
    return SettingsModule("ROI Extraction Settings",
                          multiprocessing_settings_block(main_widget),
                          roi_extraction_settings_block(main_widget),
                          roi_advanced_settings_block(main_widget), show_name=False)


if __name__ == "__main__":
    app = QApplication([])

    widget = preprocessing_settings()  # noqa
    widget.setStyleSheet(qdarkstyle.load_stylesheet())
    widget.show()

    sys.exit(app.exec_())



import sys

from qtpy import QtCore
from qtpy import QtGui
from qtpy.QtWidgets import *


class Stream(QtCore.QObject):
    newText = QtCore.Signal(str)

    def __init__(self):
        super().__init__()

        # print(type(self.newText))
        # self.connect(self.newText)

    def write(self, text):
        self.newText.emit(str(text))

    def flush(self):
        pass


class ConsoleWidget(QWidget):
    def __init__(self):
        super().__init__()
        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.label = QLabel()
        self.label.setText("Console:")
        self.layout.addWidget(self.label)
        self.setMinimumHeight(100)
        self.process = QTextBrowser()
        self.process.moveCursor(QtGui.QTextCursor.Start)
        self.process.ensureCursorVisible()
        self.process.setLineWrapColumnOrWidth(500)
        self.process.setLineWrapMode(QTextEdit.FixedPixelWidth)

        self.layout.addWidget(self.process)
        self.setLayout(self.layout)
        sys.stdout = Stream()
        sys.stdout.newText.connect(self.onUpdateText)

    def onUpdateText(self, text):
        cursor = self.process.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.insertText(text)
        self.process.setTextCursor(cursor)
        self.process.ensureCursorVisible()

    def __del__(self):
        sys.stdout = sys.__stdout__



import os

from qtpy import QtCore
from qtpy.QtGui import *
from qtpy.QtWidgets import *


class TrialListWidget(QWidget):
    def __init__(self):

        self.trial_paths = []
        self.trial_items = []

        super().__init__()
        self.list = QListWidget()
        self.setStyleSheet("QListView::item { border-bottom: 1px solid rgb(50, 65, " +
                           "75); }")
        self.top_labels_layout = QHBoxLayout()
        label1 = QLabel(text="Trial Selection")
        self.top_labels_layout.addWidget(label1)
        label1.setStyleSheet("font-size:20")

        self.model = QStandardItemModel(self.list)

        self.layout = QVBoxLayout()
        self.layout.setContentsMargins(2, 2, 2, 2)

        self.layout.addLayout(self.top_labels_layout)
        self.layout.addWidget(self.list)
        self.roi_item_list = []
        self.setLayout(self.layout)

    def set_items_from_path(self, path):
        """
        Takes in a  path to a folder or a single file and adds the trials to the list view
        Parameters
        ----------
        path path to folder or single tiff file

        Returns
        -------
        Nothing
        """
        self.list.clear()
        self.trial_items = []
        if os.path.isfile(path):
            self.trial_paths = [path]
        else:
            self.trial_paths = sorted(os.listdir(path))
        for path in self.trial_paths:
            self.trial_items.append(QListWidgetItem(path, self.list))
            self.trial_items[-1].setFlags(
                self.trial_items[-1].flags() | QtCore.Qt.ItemIsUserCheckable)
            self.trial_items[-1].setCheckState(QtCore.Qt.Checked)

    def set_items_from_list(self, trials, trials_selected_indices):
        """
        Adds trials to the list
        Parameters
        ----------
        trials
            a list of the trials
        trials_selected_indices
            indicies of the trials that should be selected

        Returns
        -------

        """
        self.list.clear()
        self.trial_items = []
        self.trial_paths = trials
        for path in trials:
            self.trial_items.append(QListWidgetItem(path, self.list))
            self.trial_items[-1].setFlags(
                self.trial_items[-1].flags() | QtCore.Qt.ItemIsUserCheckable)
            self.trial_items[-1].setCheckState(QtCore.Qt.Unchecked)
        for indice in trials_selected_indices:
            self.trial_items[indice].setCheckState(QtCore.Qt.Checked)

    def selectedTrials(self):
        return [self.trial_paths[x[0]] for x in enumerate(self.trial_items)
                if x[1].checkState() == Qt.CheckState.Checked]



from qtpy.QtWidgets import *

from CIDAN.GUI.Inputs.Input import Input


class FileInput(Input):
    def __init__(self, display_name, program_name, on_change_function, default_val,
                 tool_tip, isFolder, forOpen):
        super().__init__(display_name, program_name, on_change_function,
                         default_val, tool_tip)
        self.isFolder = isFolder
        self.forOpen = forOpen
        self.path = ""
        self.current_location = QLabel()
        self.current_location.setText("")
        self.layout_h.addWidget(self.current_location)
        self.button = QPushButton()
        self.button.setText("Browse")
        self.button.clicked.connect(self.on_browse_button)
        self.button.setToolTip(tool_tip)
        self.layout_h.addWidget(self.button)

    def on_browse_button(self):
        self.path = createFileDialog(directory="~/Desktop", forOpen=self.forOpen,
                                     isFolder=self.isFolder)
        self.current_location.setText(self.path)
        if self.on_change_function is not None:
            self.on_change_function(self.path)

    def current_state(self):
        return self.path


def createFileDialog(directory='', forOpen=True, fmt='', isFolder=0):
    directory = "/Users/sschickler/Documents/LSSC-python/input_images"
    options = QFileDialog.Options()
    options |= QFileDialog.DontUseNativeDialog
    options |= QFileDialog.DontUseCustomDirectoryIcons
    dialog = QFileDialog()
    dialog.setOptions(options)

    dialog.setFilter(dialog.filter())

    # ARE WE TALKING ABOUT FILES OR FOLDERS
    if isFolder == 1:
        dialog.setFileMode(QFileDialog.DirectoryOnly)
    if isFolder == 2:
        dialog.setFileMode(QFileDialog.AnyFile)
    # OPENING OR SAVING
    dialog.setAcceptMode(
        QFileDialog.AcceptOpen) if forOpen else dialog.setAcceptMode(
        QFileDialog.AcceptSave)

    # SET FORMAT, IF SPECIFIED
    if fmt != '' and isFolder is False:
        dialog.setDefaultSuffix(fmt)
        dialog.setNameFilters([f'{fmt} (*.{fmt})'])

    # SET THE STARTING DIRECTORY
    if directory != '':
        dialog.setDirectory(str(directory))
    # else:
    #     dialog.setDirectory(str(ROOT_DIR))

    if dialog.exec_() == QDialog.Accepted:
        path = dialog.selectedFiles()[0]  # returns a list
        return path
    else:
        return ''



import logging
import sys

from CIDAN.GUI.Data_Interaction.Thread import Thread

logger = logging.getLogger("CIDAN.GUI.Data_Interaction.PreprocessThread")


class PreprocessThread(Thread):
    def __init__(self, main_widget, button, preprocess_tab):
        super().__init__(main_widget.data_handler)
        self.main_widget = main_widget
        self.button = button
        self.preprocess_tab = preprocess_tab
        self.signal.sig.connect(lambda x: self.endThread(x))

    def run(self):
        try:
            self.data_handler.calculate_filters()
            self.signal.sig.emit(True)
        except Exception as e:
            logger.error(e)
            print("Unexpected error:", sys.exc_info()[0])
            self.signal.sig.emit(False)

    def runThread(self):

        if not any([x.isRunning() for x in self.main_widget.thread_list]):
            print("Starting preprocessing sequence")
            # self.button.setEnabled(False)
            self.start()
        else:
            print(
                "Previous process in process, please wait to start new one till "
                "finished")

    def endThread(self, success):
        self.button.setEnabled(True)
        if success:
            print("Finished preprocessing sequence")
            self.preprocess_tab.set_image_display_list(self.data_handler.trials_loaded,
                                                       self.data_handler.dataset_trials_filtered_loaded)



import numpy as np
from dask import array


def calculate_temporal_correlation(dataset):
    shape = dataset.shape
    default_val = 0  # np.average(dataset)
    dataset_flat = array.from_array(
        dataset.transpose([1, 2, 0]).reshape([shape[1] * shape[2], -1]))

    corr_up = test_func(default_val, dataset_flat, shape, dataset).compute()
    shift_down = np.hstack(
        [dataset[:, 1:, :], np.full((shape[0], 1, shape[2]), default_val)]).transpose(
        [1, 2, 0]).reshape([shape[1] * shape[2], -1])
    corr_down = np.average(dataset_flat * shift_down, axis=1)
    del shift_down
    shift_right = np.dstack(
        [np.full((shape[0], shape[1], 1), default_val), dataset[:, :, :-1]]).transpose(
        [1, 2, 0]).reshape([shape[1] * shape[2], -1])
    corr_right = np.average(dataset_flat * shift_right, axis=1)
    del shift_right
    shift_left = np.dstack(
        [dataset[:, :, :-1], np.full((shape[0], shape[1], 1), default_val)]).transpose(
        [1, 2, 0]).reshape([shape[1] * shape[2], -1])
    corr_left = np.average(dataset_flat * shift_left, axis=1)
    del shift_left
    avg_correlation = np.average(np.vstack([corr_up, corr_down, corr_left, corr_right]),
                                 axis=0)
    avg_correlation_image = avg_correlation.reshape([shape[1], shape[2]])
    return avg_correlation_image


def test_func(default_val, dataset_flat, shape, dataset):
    shift_up = array.hstack(
        [array.zeros((shape[0], 1, shape[2])), dataset[:, :-1, :]]).transpose(
        [1, 2, 0]).reshape([shape[1] * shape[2], -1])

    shift_up_mult = dataset_flat * shift_up
    del shift_up
    return array.mean(shift_up_mult, axis=1)



import numpy as np


def calculateMeanTrace(roi, data):
    """
    Calcualtes the mean time trace for a given roi and data
    Parameters
    ----------
    roi list of pixels for data (1D cords)
    data 2D list of data for a single trials with pixel num as dim 0, time as dim 1

    Returns
    -------
    A 1d nd array with the time trace for the ROI
    """
    return np.mean(data[roi], axis=0)



import numpy as np
def calculateDeltaFOverF(roi, data):
    """
    Calcualtes the deltaf/f time trace for a given roi and data
    Parameters
    ----------
    roi list of pixels for data (1D cords)
    data 2D list of data for a single trials with pixel num as dim 0, time as dim 1

    Returns
    -------
    A 1d nd array with the time trace for the ROI
    """
    pixels_in_cluster = data[roi]
    pixel_sum = np.sum(pixels_in_cluster, axis=1)
    bottom_10_number = np.percentile(pixel_sum, 10)
    bottom_10_pixels = pixels_in_cluster[pixel_sum < bottom_10_number]
    bottom_10_avg = np.mean(bottom_10_pixels, axis=0)
    return (np.mean(pixels_in_cluster, axis=0) - bottom_10_avg) / bottom_10_avg



from qtpy import QtCore
from qtpy.QtWidgets import *

from CIDAN.GUI.Data_Interaction.PreprocessThread import PreprocessThread
from CIDAN.GUI.Inputs.OptionInput import OptionInput
from CIDAN.GUI.SettingWidget.SettingsModule import preprocessing_settings
from CIDAN.GUI.Tabs.Tab import Tab


class PreprocessingTab(Tab):
    """Class controlling the Preprocessing tab, inherits from Tab


    Attributes
    ----------
    main_widget : MainWidget
        A reference to the main widget
    data_handler : DataHandler
        A reference to the main DataHandler of MainWidget

    """

    def __init__(self, main_widget):
        self.main_widget = main_widget
        self.data_handler = self.main_widget.data_handler

        # This part initializes the button to process the data
        process_button = QPushButton()
        process_button.setText("Apply Settings")
        process_button_layout = QVBoxLayout()
        process_button_widget = QWidget()
        process_button_widget.setLayout(process_button_layout)
        process_button_layout.addWidget(process_button)
        process_button_layout.setContentsMargins(2, 2, 2, 2)
        thread = PreprocessThread(main_widget, process_button, self)
        main_widget.thread_list.append(thread)  # Appends the thread to the main
        # widget thread list
        process_button.clicked.connect(lambda: thread.runThread())
        # This assumes that the data is already loaded in
        self.data_handler.calculate_filters()

        # Section that creates all the buttons to change which image is displayed
        image_buttons = QWidget()
        self._image_buttons_layout = QHBoxLayout()
        self._image_buttons_layout.setContentsMargins(2, 0, 2, 0)
        image_buttons.setLayout(self._image_buttons_layout)
        max_image_button = QPushButton()
        max_image_button.setText("Max Image")
        max_image_button.clicked.connect(
            lambda: self.set_image_display(
                self.data_handler.max_image))
        stack_button = QPushButton()
        stack_button.setText("Filtered Stack")
        stack_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.dataset_trials_filtered_loaded))
        orig_stack_button = QPushButton()
        orig_stack_button.setText("Original Stack")
        orig_stack_button.clicked.connect(
            lambda: self.set_image_display_list(self.data_handler.trials_loaded,
                                                self.data_handler.dataset_trials_loaded))
        mean_image_button = QPushButton()
        mean_image_button.setText("Mean Image")
        mean_image_button.clicked.connect(
            lambda: self.set_image_display(
                self.data_handler.mean_image))
        self._image_buttons_layout.addWidget(orig_stack_button)
        self._image_buttons_layout.addWidget(stack_button)
        self._image_buttons_layout.addWidget(max_image_button)
        self._image_buttons_layout.addWidget(mean_image_button)
        self.set_image_display_list(self.data_handler.trials_loaded,
                                    self.data_handler.dataset_trials_filtered_loaded)

        main_widget.preprocess_image_view.setContentsMargins(0, 0, 0, 0)
        # main_widget.preprocess_image_view.setMargin(0)
        preprocessing_settings_widget = preprocessing_settings(main_widget)
        preprocessing_settings_widget.setContentsMargins(0, 0, 0, 0)
        # Initialize the tab with the necessary columns
        super().__init__("Preprocessing", column_1=[preprocessing_settings_widget
                                                    ],
                         column_2=[main_widget.preprocess_image_view
                                   ])
        self.column_1_layout.addWidget(process_button_widget,
                                       alignment=QtCore.Qt.AlignBottom)
        self.column_2_layout.addWidget(image_buttons, alignment=QtCore.Qt.AlignBottom)

    def set_image_display_list(self, trial_names, data_list):
        """
        Sets the preprocessing image display to use an option input and set data list
        Parameters
        ----------
        trial_names : List[str]
            the names of each trial
        data_list : List[np.ndarray]
            Corresponding data for each trial
        Returns
        -------
        Nothing
        """

        def set_image(x, trial_name):
            self.main_widget.preprocess_image_view.setImage(
                data_list[trial_names.index(trial_name)])

        if hasattr(self, "trial_selector_input"):
            self.trial_selector_input.setParent(None)
        self.trial_selector_input = OptionInput("", "", set_image, val_list=trial_names,
                                                tool_tip="Select Trial to display",
                                                display_tool_tip=False, default_index=0,
                                                show_name=False)
        self._image_buttons_layout.addWidget(self.trial_selector_input)
        set_image("", trial_names[0])

    def set_image_display(self, data):
        self.trial_selector_input.setParent(None)
        self.trial_selector_input = OptionInput("", "", lambda x, y: 3,
                                                val_list=["All"],
                                                tool_tip="Select Trial to display",
                                                display_tool_tip=False, default_index=0,
                                                show_name=False)
        self._image_buttons_layout.addWidget(self.trial_selector_input)
        self.main_widget.preprocess_image_view.setImage(
            data)



import os

os.environ["OMP_NUM_THREADS"] = "10"  # export OMP_NUM_THREADS=4
os.environ["OPENBLAS_NUM_THREADS"] = "10"  # export OPENBLAS_NUM_THREADS=4
os.environ["MKL_NUM_THREADS"] = "10"  # export MKL_NUM_THREADS=6
os.environ["VECLIB_MAXIMUM_THREADS"] = "10"  # export VECLIB_MAXIMUM_THREADS=4
os.environ["NUMEXPR_NUM_THREADS"] = "10"  # export NUMEXPR_NUM_THREADS=6
import scipy.sparse as sparse
import hnswlib as hnsw
import numpy as np
from typing import Tuple

from dask import delayed


@delayed
def calcAffinityMatrix(*, pixel_list: np.ndarray, metric: str, knn: int,
                       accuracy: int, connections: int, normalize_w_k: int,
                       num_threads:
                       int, spatial_box_num: int, temporal_box_num: int):
    """
    Calculates an pairwise affinity matrix for the image stack
    Parameters
    ----------
    pixel_list: a 2d np array of pixels with dim 0 being a list of pixels and dim 1 being pixel values over time
    metric: can be "l2" squared l2, "ip" Inner product, "cosine" Cosine similarity
    knn: number of nearest neighbors to search for
    accuracy: time of construction vs acuracy tradeoff
    connections: max number of outgoing connections
    num_threads: number of threads to use
    normalize_w_k: kth clostest neighbor for autotune

    Returns
    -------
    affinity matrix
    """
    # TODO make connections value scale based on available memory

    dim = pixel_list.shape[1]
    num_elements = pixel_list.shape[0]
    print("Spatial Box {}, Time Step {}: Started Processing".format(spatial_box_num,
                                                                    temporal_box_num))
    knn_graph = hnsw.Index(space=metric, dim=dim)
    knn_graph.init_index(max_elements=num_elements, ef_construction=accuracy,
                         M=connections)
    knn_graph.add_items(pixel_list, num_threads=num_threads)
    indices, distances = knn_graph.knn_query(pixel_list, k=knn,
                                             num_threads=num_threads)
    # lazy random walk means it returns distance of zero for same point
    # nbrs = LSHForest(n_estimators=20, n_candidates=200,
    #                  n_neighbors=10).fit(pixel_list)
    # distances, indices = nbrs.kneighbors(pixel_list)
    # TODO add comments here
    reformat_indicies_x = np.repeat(np.arange(0, num_elements, 1), knn-1)
    reformat_indicies_y = np.reshape(indices[:,1:], (-1))
    reformat_distances = np.reshape(distances[:,1:], (-1))
    # Self tuning adaptive bandwidth
    scale_factor_indices = np.repeat(distances[:, normalize_w_k], knn)
    scale_factor_2_per_distances = np.power(scale_factor_indices[reformat_indicies_x],
                                            .5) * \
                                   np.power(scale_factor_indices[reformat_indicies_y],
                                            .5)
    reformat_distances_scaled = np.exp(
        -reformat_distances / scale_factor_2_per_distances)
    # TODO change to go direct to csr matrix
    return sparse.csr_matrix(sparse.coo_matrix(
        (
            np.hstack([reformat_distances_scaled,np.ones((num_elements))]),
            (np.hstack([reformat_indicies_x,np.arange(0, num_elements, 1)]),np.hstack( [reformat_indicies_y,np.arange(0, num_elements, 1)]))),
        shape=(num_elements, num_elements)))


def calcDInv(K: sparse.csr_matrix):
    """Calculates a scaling diagonal matrix D to rescale eigen vectors

    Parameters
    ----------
    K the sparse matrix K for the pairwise affinity

    Returns
    -------
    a sparse matrix with type csr, and D's diagonal values
    """
    dim = K.shape[0]
    D_diag = np.nan_to_num(1/K.sum(axis=1), nan=0.000001, posinf =0.0000001, neginf=0.0000001) #add small epsilon to each row in K.sum()
    # D_diag = 1 / K.sum(axis=1)
    # print("D_diag",D_diag)
    D_sparse = sparse.dia_matrix((np.reshape(D_diag, [1, -1]), [0]),
                                 (dim, dim))
    return sparse.csr_matrix(D_sparse), D_diag


def calcLaplacian(P_sparse: sparse.csr_matrix, dim: Tuple[int, int]):
    """
    Calculates the Laplacian
    Parameters
    ----------
    P_sparse
        A sparse matrix of the normalized affinity matrix
    dim
        dimensions of P_sparse
    Returns
    -------
    Laplacian matrix
    # TODO I'm a little unsure what are the dimensions here
    """
    I_sparse = sparse.identity(dim, format="csr")
    laplacian_sparse = I_sparse - P_sparse
    return laplacian_sparse


def calcDSqrt(D_diag: np.ndarray):
    """
    Calculates sparse matrix of the sqrt of D
    Parameters
    ----------
    D_diag
        A list of the diagonal indices of D a the normalization matrix for P

    Returns
    -------
    A sparse CSR matrix of the sqrt of D
    """
    dim = D_diag.shape[0]
    D_sqrt = sparse.csr_matrix(
        sparse.dia_matrix((np.reshape(np.power(D_diag, .5), [1, -1]), [0]),
                          (dim, dim)))
    return D_sqrt


def calcDNegSqrt(D_diag):
    """
    Calculates sparse matrix of the neg sqrt of D
    Parameters
    ----------
    D_diag
        A list of the diagonal indices of D a the normalization matrix for P

    Returns
    -------
    A sparse CSR matrix of the neg sqrt of D
    """
    dim = D_diag.shape[0]
    D_neg_sqrt = sparse.csr_matrix(
        sparse.dia_matrix(
            (np.reshape(np.power(D_diag, -.5), [1, -1]), [0]),
            (dim, dim)))
    return D_neg_sqrt


def embedEigenSqrdNorm(eigen_vectors: np.ndarray) -> np.ndarray:
    """
    Embeds all pixels in image to a set of eigen vectors
    Parameters
    ----------
    eigen_vectors A set of eigen vectors

    Returns
    -------
    A list of pixel values if they where represented by those eigen vectors
    """
    pixel_embedings = np.sum(np.power(eigen_vectors, 2),
                             axis=1)
    return pixel_embedings



from typing import List

from CIDAN.GUI.ListWidgets.ROIListModule import *


class Column(QWidget):  # just used for stylesheets
    pass


class Tab(QWidget):
    def __init__(self, name, column_1: List[QFrame], column_2: List[QFrame],
                 column_2_display=True):
        super().__init__()
        self.name = name
        self.column_1 = column_1
        self.column_2 = column_2
        self.setMinimumHeight(500)
        self.layout = QHBoxLayout()  # Main layout class
        self.layout.setContentsMargins(2, 2, 2, 2)
        self.column_1_widget = Column()

        self.column_1_layout = QVBoxLayout()  # Layout for column 1
        self.column_1_layout.setContentsMargins(0, 0, 0, 0)
        self.column_1_widget.setLayout(self.column_1_layout)
        self.column_1_widget.setStyleSheet("Column { border:1px solid rgb(50, 65, "
                                           "75);} ")
        for module in column_1:
            self.column_1_layout.addWidget(module)
        self.layout.addWidget(self.column_1_widget, stretch=1)
        if column_2_display:
            self.column_2_layout = QVBoxLayout()  # Layout for column 2
            self.column_2_layout.setContentsMargins(0, 0, 0, 0)

            self.column_2_widget = Column()
            self.column_2_widget.setStyleSheet("Column { border:1px solid rgb(50, 65, "
                                               "75);} ")
            self.column_2_widget.setLayout(self.column_2_layout)
            for module in column_2:
                self.column_2_layout.addWidget(module)
            self.layout.addWidget(self.column_2_widget, stretch=2)
        self.setLayout(self.layout)


class AnalysisTab(Tab):
    def __init__(self, main_widget):
        super().__init__("Analysis", column_1=[], column_2=[])



import os

os.environ['QT_API'] = 'pyside2'
from qtpy.QtWidgets import QTabWidget
from CIDAN.GUI.Tabs.Tab import AnalysisTab
from CIDAN.GUI.Tabs.FileOpenTab import FileOpenTab
from CIDAN.GUI.Tabs.ROIExtractionTab import *
from CIDAN.GUI.Tabs.PreprocessingTab import *
import qdarkstyle
from CIDAN.GUI.ImageView.ImageViewModule import ImageViewModule
from CIDAN.GUI.Data_Interaction.DataHandler import DataHandler
from CIDAN.GUI.Console.ConsoleWidget import ConsoleWidget
import sys
import logging


class MainWindow(QMainWindow):
    """Initializes the basic window with Main widget being the focused widget"""

    def __init__(self, dev=False):
        super().__init__()
        self.title = 'CIDAN'
        self.width = 900
        self.height = 800
        self.setWindowTitle(self.title)
        self.setMinimumSize(self.width, self.height)
        self.main_menu = self.menuBar()
        self.table_widget = MainWidget(self, dev=dev)
        self.setCentralWidget(self.table_widget)
        # self.setStyleSheet(qdarkstyle.load_stylesheet())
        style = """
            QTabWidget {font-size: 25px; padding:1px; margin:5px;}
            QTabBar::tab {
                background: qlineargradient(x1: 0, y1: 0, x2: 0, y2: 1,
                                           stop: 0 #E1E1E1, stop: 0.4 #DDDDDD,
                                          stop: 0.5 #D8D8D8, stop: 1.0 #D3D3D3);
                /*border: 2px solid #C4C4C3;*/
                /*border-bottom-color: #C2C7CB; !* same as the pane color *!*/
                
                min-width: 8ex;
                padding:1px;
                border:1px;
            }
            
            QComboBox::item:checked {
              font-weight: bold;
              height: 12px;
            }
            """
        self.setStyleSheet(qdarkstyle.load_stylesheet() + style)

        # extractAction.triggered.connect()

        self.show()


class MainWidget(QWidget):
    """Main Widget, contains everything

    Attributes
    ----------
    main_window : MainWindow
        A reference to the main window of the application
    main_menu : ???
        the top bar menu
    layout : QLayout
        The main layout for the widget
    data_handler : DataHandler
        The instance that controls all interactions with dataset
    thread_list : List[Thread]
        A list of all the possible running threads, used to ensure only 1 thread is
        running at a time
    preprocess_image_view : ImageViewModule
        The image view for the preprocess tab
    roi_image_view : ImageViewModule
        The image view for the roi extraction tab
    tab_widget : QTabWidget
        Controls the main tabs of the application
    console : ConsoleWidget
        Widget for the console
    tabs : List[Tabs]
        A list of the currently active tabs not used until after init_w_data is run
    """

    def __init__(self, parent, dev=False):
        """
        Initialize the main widget to load files
        Parameters
        ----------
        parent
        """
        super().__init__(parent)
        self.main_window = parent
        self.main_menu = self.main_window.main_menu
        self.layout = QVBoxLayout(self)
        self.data_handler = None
        self.thread_list = []
        self.preprocess_image_view = ImageViewModule(self, roi=False)
        self.roi_image_view = ImageViewModule(self, histogram=False, roi=False)
        self.dev = dev
        self.tab_widget = QTabWidget()
        self.fileOpenTab = FileOpenTab(self)
        self.tab_widget.addTab(self.fileOpenTab, "Open Dataset")

        # This part add placeholder tabs until data is loaded
        self.tabs = ["Preprocessing", "ROI Extraction", "Analysis"]
        for num, tab in enumerate(self.tabs):
            self.tab_widget.addTab(QWidget(), tab)
            self.tab_widget.setTabEnabled(num + 1, False)
        self.layout.addWidget(self.tab_widget)

        self.console = ConsoleWidget()
        self.console.setMaximumHeight(150)
        self.console.setMinimumHeight(150)
        self.layout.addWidget(self.console)
        self.setLayout(self.layout)

        # Initialize top bar menu
        fileMenu = self.main_menu.addMenu('&File')
        openFileAction = QAction("Open File", self)
        openFileAction.setStatusTip('Open a single file')
        openFileAction.triggered.connect(lambda: self.selectOpenFileTab(0))
        fileMenu.addAction(openFileAction)
        openFolderAction = QAction("Open Folder", self)
        openFolderAction.setStatusTip('Open a folder')
        openFolderAction.triggered.connect(lambda: self.selectOpenFileTab(1))
        fileMenu.addAction(openFolderAction)
        openPrevAction = QAction("Open Previous Session", self)
        openPrevAction.setStatusTip('Open a previous session')
        openPrevAction.triggered.connect(lambda: self.selectOpenFileTab(2))
        fileMenu.addAction(openPrevAction)

        # Below here in this function is just code for testing
        # TODO check if it can load data twice
        if False and dev:
            # auto loads a small dataset
            self.data_handler = DataHandler(

                "/Users/sschickler/Code Devel/LSSC-python/input_images/",
                "C:\\Users\\gadge\\Downloads\\dELETE",
                trials=["small_dataset.tif"],
                save_dir_already_created=True)
            self.init_w_data()
        if False and dev:
            # auto loads a large dataset
            self.data_handler = DataHandler(
                "/Users/sschickler/Code Devel/LSSC-python/input_images/dataset_1",
                "/Users/sschickler/Code Devel/LSSC-python/input_images/test3",
                save_dir_already_created=False)
            self.init_w_data()

    def init_w_data(self):
        """
        Initialize main widget with data

        It activates the other tabs and helps load the data into image views
        Returns
        -------

        """
        for num, _ in enumerate(self.tabs):
            self.tab_widget.removeTab(1)
        # TODO actually delete the tabs not just remove them
        # TODO add to export tab to export all time traces or just currently caclulated ones
        self.tabs = [PreprocessingTab(self), ROIExtractionTab(self), AnalysisTab(self)]

        # Add tabs
        for tab in self.tabs:
            self.tab_widget.addTab(tab, tab.name)
        self.tab_widget.setCurrentIndex(1)
        self.tab_widget.currentChanged.connect(
            lambda x: self.tabs[1].set_background("",
                                                  self.tabs[
                                                      1].background_chooser.current_state(),
                                                  update_image=True))

        if not hasattr(self, "export_menu"):
            self.export_menu = self.main_menu.addMenu("&Export")
            export_action = QAction("Export Time Traces/ROIs", self)
            export_action.setStatusTip('Export Time Traces/ROIs')
            export_action.triggered.connect(lambda: self.exportStuff())
            self.export_menu.addAction(export_action)

    def selectOpenFileTab(self, index):
        self.tab_widget.setCurrentIndex(0)
        self.fileOpenTab.tab_selector.setCurrentIndex(index)

    def exportStuff(self):
        msg = QMessageBox()
        msg.setWindowTitle("Export data")
        msg.setText("Data Exported to save directory")
        msg.setIcon(QMessageBox.Information)
        x = msg.exec_()


if __name__ == "__main__":
    # client = Client(processes=False, threads_per_worker=8,
    #                 n_workers=1, memory_limit='16GB')
    # print(client)
    LOG_FILENAME = 'log.out'
    logging.basicConfig(filename=LOG_FILENAME, level=logging.DEBUG)
    logger = logging.getLogger("CIDAN")
    logger.debug("Program started")
    app = QApplication([])
    app.setApplicationName("CIDAN")
    widget = MainWindow(dev=True)

    sys.exit(app.exec_())



from qtpy.QtWidgets import *

from CIDAN.GUI.Inputs.Input import Input


class OptionInput(Input):
    def __init__(self, display_name, program_name, on_change_function, default_index,
                 tool_tip,
                 val_list, display_tool_tip=False, show_name=True):
        super().__init__(display_name, program_name, on_change_function,
                         default_index, tool_tip, display_tool_tip, show_name=show_name)

        self.input_box = QComboBox()
        self.val_list = val_list
        self.input_box.addItems(val_list)

        self.input_box.setCurrentIndex(self.default_val)
        self.input_box.setToolTip(self.tool_tip)
        self.input_box.currentIndexChanged.connect(self.on_change)
        self.layout_h.addWidget(self.input_box)

    def current_state(self):
        return self.val_list[self.input_box.currentIndex()]

    def set_default_val(self):
        self.input_box.setCurrentIndex(self.default_val)



from setuptools import setup, find_packages

with open("README.md", 'r') as f:
    long_description = f.read()

setup(
    name='CIDAN',
    version='0.1.13',
    description='CIDAN-Calcium Imaging Data ANalysis',
    license="MIT",
    long_description=long_description,
    author='Sam Schickler',
    author_email='sschickl@ucsd.edu',
    # url="http://www.foopackage.com/",
    packages=find_packages(),
    install_requires=["numpy", "QtPy", "QDarkStyle", "pybind11", "pyqtgraph==0.11.0rc0",
                      "Pyside2",
                      "dask[complete]", "matplotlib", "scipy", "tiffile",
                      "scikit-image", "hnswlib", "pillow",
                      'tifffile'],
    scripts=[
        "CIDAN/LSSC/process_data.py",
        "CIDAN/TimeTrace/deltaFOverF.py",
        "CIDAN/TimeTrace/mean.py",
        "CIDAN/LSSC/SpatialBox.py",
        "CIDAN/TimeTrace/__init__.py",
        "CIDAN/LSSC/functions/roi_filter.py",
        'CIDAN/LSSC/functions/data_manipulation.py',
        "CIDAN/LSSC/functions/eigen.py",
        "CIDAN/LSSC/functions/embeddings.py",
        "CIDAN/LSSC/functions/pickle_funcs.py",
        "CIDAN/LSSC/functions/roi_extraction.py",
        "CIDAN/LSSC/functions/save_test_images.py",
        "CIDAN/LSSC/functions/temporal_correlation.py",
        "CIDAN/GUI/Console/__init__.py",
        "CIDAN/GUI/Console/ConsoleWidget.py",
        "CIDAN/GUI/Data_Interaction/__init__.py",
        "CIDAN/GUI/Data_Interaction/DataHandler.py",
        "CIDAN/GUI/Data_Interaction/PreprocessThread.py",
        "CIDAN/GUI/Data_Interaction/ROIExtractionThread.py",
        "CIDAN/GUI/Data_Interaction/Signals.py",
        "CIDAN/GUI/Data_Interaction/Thread.py",
        "CIDAN/GUI/Data_Interaction/loadDataset.py",
        "CIDAN/GUI/ImageView/__init__.py",
        "CIDAN/GUI/ImageView/ImageViewModule.py",
        "CIDAN/GUI/Inputs/__init__.py",
        "CIDAN/GUI/Inputs/Input.py",
        "CIDAN/GUI/Inputs/BoolInput.py",
        "CIDAN/GUI/Inputs/FileInput.py",
        "CIDAN/GUI/Inputs/FloatInput.py",
        "CIDAN/GUI/Inputs/Int3DInput.py",
        "CIDAN/GUI/Inputs/IntInput.py",
        "CIDAN/GUI/Inputs/OptionInput.py",
        "CIDAN/GUI/ListWidgets/__init__.py",
        "CIDAN/GUI/ListWidgets/ROIItemModule.py",
        "CIDAN/GUI/ListWidgets/ROIItemWidget.py",
        "CIDAN/GUI/ListWidgets/ROIListModule.py",
        "CIDAN/GUI/ListWidgets/TrialListWidget.py",
        "CIDAN/GUI/SettingWidget/__init__.py",
        "CIDAN/GUI/SettingWidget/SettingBlockModule.py",
        "CIDAN/GUI/SettingWidget/SettingsModule.py",
        'CIDAN/GUI/Tabs/__init__.py',
        "CIDAN/GUI/Tabs/FileOpenTab.py",
        "CIDAN/GUI/Tabs/PreprocessingTab.py",
        "CIDAN/GUI/Tabs/ROIExtractionTab.py",
        "CIDAN/GUI/Tabs/Tab.py",
        "CIDAN/__init__.py",
        "CIDAN/__main__.py"
        ]
)



from qtpy.QtWidgets import *

from CIDAN.GUI.Inputs.Input import Input


class IntRangeInput(Input):
    def __init__(self, display_name, program_name, on_change_function, default_val,
                 tool_tip,
                 min, max, step, display_tool_tip=False
                 ):
        super().__init__(display_name, program_name, on_change_function, default_val,
                         tool_tip, display_tool_tip)

        self.input_box_1 = QSpinBox()
        self.input_box_1.setMinimum(min)
        self.input_box_1.setMaximumWidth(50)
        self.input_box_1.setMaximum(max)
        self.input_box_1.setSingleStep(step)
        self.input_box_1.setValue(self.default_val[0])
        self.input_box_1.setToolTip(self.tool_tip)
        self.layout_h.addWidget(self.input_box_1)
        self.layout_h.addWidget(QLabel(" to "))
        self.input_box_1.valueChanged.connect(self.on_change)
        self.input_box_2 = QSpinBox()
        self.input_box_2.setMinimum(min)
        self.input_box_2.setMaximumWidth(50)
        self.input_box_2.setMaximum(max)
        self.input_box_2.setSingleStep(step)
        self.input_box_2.setValue(self.default_val[1])
        self.input_box_2.setToolTip(self.tool_tip)
        self.layout_h.addWidget(self.input_box_2)
        self.input_box_2.valueChanged.connect(self.on_change)

    def current_state(self):
        return [self.input_box_1.value(), self.input_box_2.value()]

    def set_default_val(self):
        self.input_box_1.setValue(self.default_val)
        self.input_box_2.setValue(self.default_val)



from qtpy.QtWidgets import *

from CIDAN.GUI.Inputs.Input import Input


class BoolInput(Input):
    def __init__(self, display_name, program_name, on_change_function, default_val,
                 tool_tip, display_tool_tip=False):
        super().__init__(display_name, program_name, on_change_function, default_val,
                         tool_tip, display_tool_tip)

        self.input_box = QRadioButton()
        self.input_box.setMaximumWidth(50)
        self.input_box.setChecked(self.default_val)
        self.input_box.toggled.connect(self.on_change)
        self.input_box.setToolTip(self.tool_tip)
        self.layout_h.addWidget(self.input_box)

    def current_state(self):
        return self.input_box.isChecked()

    def set_default_val(self):
        self.input_box.setChecked(self.default_val)
